@@ -6,15 +6,25 @@ This is not a public API.
 # necessary to enforce truediv in Python 2.X
 from __future__ import division
 import operator
+import warnings
 import numpy as np
 import pandas as pd
+import datetime
 from pandas import compat, lib, tslib
 import pandas.index as _index
 from pandas.util.decorators import Appender
 import pandas.core.common as com
 import pandas.computation.expressions as expressions
-from pandas.core.common import(bind_method, is_list_like, notnull, isnull,
-                               _values_from_object, _maybe_match_name)
+from pandas.lib import isscalar
+from pandas.tslib import iNaT
+from pandas.compat import bind_method
+from pandas.core.common import(is_list_like, notnull, isnull,
+                               _values_from_object, _maybe_match_name,
+                               needs_i8_conversion, is_datetimelike_v_numeric,
+                               is_integer_dtype, is_categorical_dtype, is_object_dtype,
+                               is_timedelta64_dtype, is_datetime64_dtype, is_datetime64tz_dtype,
+                               is_bool_dtype)
+from pandas.io.common import PerformanceWarning
 
 # -----------------------------------------------------------------------------
 # Functions that add arithmetic methods to objects, given arithmetic factory
@@ -257,75 +267,95 @@ class _TimeOp(object):
     Generally, you should use classmethod ``maybe_convert_for_time_op`` as an
     entry point.
     """
-    fill_value = tslib.iNaT
+    fill_value = iNaT
     wrap_results = staticmethod(lambda x: x)
     dtype = None
 
-    def __init__(self, left, right, name):
-        self.name = name
+    def __init__(self, left, right, name, na_op):
 
         # need to make sure that we are aligning the data
         if isinstance(left, pd.Series) and isinstance(right, pd.Series):
             left, right = left.align(right,copy=False)
 
-        self.left = left
-        self.right = right
         lvalues = self._convert_to_array(left, name=name)
         rvalues = self._convert_to_array(right, name=name, other=lvalues)
 
-        self.is_timedelta_lhs = com.is_timedelta64_dtype(left)
-        self.is_datetime_lhs = com.is_datetime64_dtype(left)
+        self.name = name
+        self.na_op = na_op
+
+        # left
+        self.left = left
+        self.is_offset_lhs = self._is_offset(left)
+        self.is_timedelta_lhs = is_timedelta64_dtype(lvalues)
+        self.is_datetime64_lhs = is_datetime64_dtype(lvalues)
+        self.is_datetime64tz_lhs = is_datetime64tz_dtype(lvalues)
+        self.is_datetime_lhs = self.is_datetime64_lhs or self.is_datetime64tz_lhs
         self.is_integer_lhs = left.dtype.kind in ['i', 'u']
-        self.is_datetime_rhs = com.is_datetime64_dtype(rvalues)
-        self.is_timedelta_rhs = com.is_timedelta64_dtype(rvalues)
+
+        # right
+        self.right = right
+        self.is_offset_rhs = self._is_offset(right)
+        self.is_datetime64_rhs = is_datetime64_dtype(rvalues)
+        self.is_datetime64tz_rhs = is_datetime64tz_dtype(rvalues)
+        self.is_datetime_rhs = self.is_datetime64_rhs or self.is_datetime64tz_rhs
+        self.is_timedelta_rhs = is_timedelta64_dtype(rvalues)
         self.is_integer_rhs = rvalues.dtype.kind in ('i', 'u')
 
-        self._validate()
+        self._validate(lvalues, rvalues, name)
+        self.lvalues, self.rvalues = self._convert_for_datetime(lvalues, rvalues)
 
-        self._convert_for_datetime(lvalues, rvalues)
-
-    def _validate(self):
+    def _validate(self, lvalues, rvalues, name):
         # timedelta and integer mul/div
 
-        if (self.is_timedelta_lhs and self.is_integer_rhs) or\
-           (self.is_integer_lhs and self.is_timedelta_rhs):
+        if (self.is_timedelta_lhs and self.is_integer_rhs) or (
+            self.is_integer_lhs and self.is_timedelta_rhs):
 
-            if self.name not in ('__truediv__', '__div__', '__mul__'):
+            if name not in ('__div__', '__truediv__', '__mul__'):
                 raise TypeError("can only operate on a timedelta and an "
                                 "integer for division, but the operator [%s]"
-                                "was passed" % self.name)
+                                "was passed" % name)
 
         # 2 datetimes
         elif self.is_datetime_lhs and self.is_datetime_rhs:
-            if self.name != '__sub__':
+
+            if name not in ('__sub__','__rsub__'):
                 raise TypeError("can only operate on a datetimes for"
                                 " subtraction, but the operator [%s] was"
-                                " passed" % self.name)
+                                " passed" % name)
+
+            # if tz's must be equal (same or None)
+            if getattr(lvalues,'tz',None) != getattr(rvalues,'tz',None):
+                raise ValueError("Incompatbile tz's on datetime subtraction ops")
 
         # 2 timedeltas
-        elif self.is_timedelta_lhs and self.is_timedelta_rhs:
+        elif ((self.is_timedelta_lhs and
+               (self.is_timedelta_rhs or self.is_offset_rhs)) or
+              (self.is_timedelta_rhs and
+               (self.is_timedelta_lhs or self.is_offset_lhs))):
 
-            if self.name not in ('__div__', '__truediv__', '__add__',
-                                 '__sub__'):
+            if name not in ('__div__', '__rdiv__', '__truediv__', '__rtruediv__',
+                            '__add__', '__radd__', '__sub__', '__rsub__'):
                 raise TypeError("can only operate on a timedeltas for "
                                 "addition, subtraction, and division, but the"
-                                " operator [%s] was passed" % self.name)
+                                " operator [%s] was passed" % name)
 
-        # datetime and timedelta
-        elif self.is_datetime_lhs and self.is_timedelta_rhs:
+        # datetime and timedelta/DateOffset
+        elif (self.is_datetime_lhs and
+              (self.is_timedelta_rhs or self.is_offset_rhs)):
 
-            if self.name not in ('__add__', '__sub__'):
+            if name not in ('__add__', '__radd__', '__sub__'):
                 raise TypeError("can only operate on a datetime with a rhs of"
-                                " a timedelta for addition and subtraction, "
+                                " a timedelta/DateOffset for addition and subtraction,"
                                 " but the operator [%s] was passed" %
-                                self.name)
+                                name)
 
-        elif self.is_timedelta_lhs and self.is_datetime_rhs:
+        elif ((self.is_timedelta_lhs or self.is_offset_lhs)
+              and self.is_datetime_rhs):
 
-            if self.name != '__add__':
-                raise TypeError("can only operate on a timedelta and"
+            if name not in ('__add__', '__radd__'):
+                raise TypeError("can only operate on a timedelta/DateOffset and"
                                 " a datetime for addition, but the operator"
-                                " [%s] was passed" % self.name)
+                                " [%s] was passed" % name)
         else:
             raise TypeError('cannot operate on a series with out a rhs '
                             'of a series/ndarray of type datetime64[ns] '
@@ -335,9 +365,10 @@ class _TimeOp(object):
         """converts values to ndarray"""
         from pandas.tseries.timedeltas import to_timedelta
 
-        coerce = True
+        ovalues = values
         if not is_list_like(values):
             values = np.array([values])
+
         inferred_type = lib.infer_dtype(values)
 
         if inferred_type in ('datetime64', 'datetime', 'date', 'time'):
@@ -346,17 +377,24 @@ class _TimeOp(object):
             if (other is not None and other.dtype == 'timedelta64[ns]' and
                     all(isnull(v) for v in values)):
                 values = np.empty(values.shape, dtype=other.dtype)
-                values[:] = tslib.iNaT
+                values[:] = iNaT
 
             # a datelike
             elif isinstance(values, pd.DatetimeIndex):
                 values = values.to_series()
+            # datetime with tz
+            elif isinstance(ovalues, datetime.datetime) and hasattr(ovalues,'tz'):
+                values = pd.DatetimeIndex(values)
+            # datetime array with tz
+            elif com.is_datetimetz(values):
+                if isinstance(values, pd.Series):
+                    values = values._values
             elif not (isinstance(values, (np.ndarray, pd.Series)) and
-                      com.is_datetime64_dtype(values)):
+                      is_datetime64_dtype(values)):
                 values = tslib.array_to_datetime(values)
         elif inferred_type in ('timedelta', 'timedelta64'):
             # have a timedelta, convert to to ns here
-            values = to_timedelta(values, coerce=coerce)
+            values = to_timedelta(values, errors='coerce')
         elif inferred_type == 'integer':
             # py3 compat where dtype is 'm' but is an integer
             if values.dtype.kind == 'm':
@@ -366,26 +404,17 @@ class _TimeOp(object):
             elif name not in ('__truediv__', '__div__', '__mul__'):
                 raise TypeError("incompatible type for a datetime/timedelta "
                                 "operation [{0}]".format(name))
-        elif isinstance(values[0], pd.DateOffset):
-            # handle DateOffsets
-            os = np.array([getattr(v, 'delta', None) for v in values])
-            mask = isnull(os)
-            if mask.any():
-                raise TypeError("cannot use a non-absolute DateOffset in "
-                                "datetime/timedelta operations [{0}]".format(
-                                    ', '.join([com.pprint_thing(v)
-                                               for v in values[mask]])))
-            values = to_timedelta(os, coerce=coerce)
         elif inferred_type == 'floating':
-
             # all nan, so ok, use the other dtype (e.g. timedelta or datetime)
             if isnull(values).all():
                 values = np.empty(values.shape, dtype=other.dtype)
-                values[:] = tslib.iNaT
+                values[:] = iNaT
             else:
                 raise TypeError(
                     'incompatible type [{0}] for a datetime/timedelta '
                     'operation'.format(np.array(values).dtype))
+        elif self._is_offset(values):
+            return values
         else:
             raise TypeError("incompatible type [{0}] for a datetime/timedelta"
                             " operation".format(np.array(values).dtype))
@@ -393,22 +422,65 @@ class _TimeOp(object):
         return values
 
     def _convert_for_datetime(self, lvalues, rvalues):
-        mask = None
+        from pandas.tseries.timedeltas import to_timedelta
+
+        mask = isnull(lvalues) | isnull(rvalues)
+
         # datetimes require views
         if self.is_datetime_lhs or self.is_datetime_rhs:
+
             # datetime subtraction means timedelta
             if self.is_datetime_lhs and self.is_datetime_rhs:
                 self.dtype = 'timedelta64[ns]'
+            elif self.is_datetime64tz_lhs:
+                self.dtype = lvalues.dtype
+            elif self.is_datetime64tz_rhs:
+                self.dtype = rvalues.dtype
             else:
                 self.dtype = 'datetime64[ns]'
-            mask = isnull(lvalues) | isnull(rvalues)
-            lvalues = lvalues.view(np.int64)
-            rvalues = rvalues.view(np.int64)
+
+            # if adding single offset try vectorized path
+            # in DatetimeIndex; otherwise elementwise apply
+            def _offset(lvalues, rvalues):
+                if len(lvalues) == 1:
+                    rvalues = pd.DatetimeIndex(rvalues)
+                    lvalues = lvalues[0]
+                else:
+                    warnings.warn("Adding/subtracting array of DateOffsets to Series not vectorized",
+                                  PerformanceWarning)
+                    rvalues = rvalues.astype('O')
+
+                # pass thru on the na_op
+                self.na_op = lambda x, y: getattr(x,self.name)(y)
+                return lvalues, rvalues
+
+
+            if self.is_offset_lhs:
+                lvalues, rvalues = _offset(lvalues, rvalues)
+            elif self.is_offset_rhs:
+                rvalues, lvalues = _offset(rvalues, lvalues)
+            else:
+
+                # with tz, convert to UTC
+                if self.is_datetime64tz_lhs:
+                    lvalues = lvalues.tz_localize(None)
+                if self.is_datetime64tz_rhs:
+                    rvalues = rvalues.tz_localize(None)
+
+                lvalues = lvalues.view(np.int64)
+                rvalues = rvalues.view(np.int64)
 
         # otherwise it's a timedelta
         else:
+
             self.dtype = 'timedelta64[ns]'
-            mask = isnull(lvalues) | isnull(rvalues)
+
+            # convert Tick DateOffset to underlying delta
+            if self.is_offset_lhs:
+                lvalues = to_timedelta(lvalues)
+            if self.is_offset_rhs:
+                rvalues = to_timedelta(rvalues)
+
             lvalues = lvalues.astype(np.int64)
             rvalues = rvalues.astype(np.int64)
 
@@ -424,18 +496,33 @@ class _TimeOp(object):
                 rvalues = rvalues.astype(np.float64)
 
         # if we need to mask the results
-        if mask is not None:
-            if mask.any():
-                def f(x):
+        if mask.any():
+            def f(x):
+
+                # datetime64[ns]/timedelta64[ns] masking
+                try:
                     x = np.array(x, dtype=self.dtype)
-                    np.putmask(x, mask, self.fill_value)
-                    return x
-                self.wrap_results = f
-        self.lvalues = lvalues
-        self.rvalues = rvalues
+                except TypeError:
+                    x = np.array(x, dtype='datetime64[ns]')
+
+                np.putmask(x, mask, self.fill_value)
+                return x
+            self.wrap_results = f
+
+        return lvalues, rvalues
+
+
+    def _is_offset(self, arr_or_obj):
+        """ check if obj or all elements of list-like is DateOffset """
+        if isinstance(arr_or_obj, pd.DateOffset):
+            return True
+        elif is_list_like(arr_or_obj):
+            return all(isinstance(x, pd.DateOffset) for x in arr_or_obj)
+        else:
+            return False
 
     @classmethod
-    def maybe_convert_for_time_op(cls, left, right, name):
+    def maybe_convert_for_time_op(cls, left, right, name, na_op):
         """
         if ``left`` and ``right`` are appropriate for datetime arithmetic with
         operation ``name``, processes them and returns a ``_TimeOp`` object
@@ -445,16 +532,13 @@ class _TimeOp(object):
         that the data is not the right type for time ops.
         """
         # decide if we can do it
-        is_timedelta_lhs = com.is_timedelta64_dtype(left)
-        is_datetime_lhs = com.is_datetime64_dtype(left)
+        is_timedelta_lhs = is_timedelta64_dtype(left)
+        is_datetime_lhs = is_datetime64_dtype(left) or is_datetime64tz_dtype(left)
+
         if not (is_datetime_lhs or is_timedelta_lhs):
             return None
 
-        # rops are allowed. No need for special checks, just strip off
-        # r part.
-        if name.startswith('__r'):
-            name = "__" + name[3:]
-        return cls(left, right, name)
+        return cls(left, right, name, na_op)
 
 
 def _arith_method_SERIES(op, name, str_rep, fill_zeros=None,
@@ -485,12 +569,12 @@ def _arith_method_SERIES(op, name, str_rep, fill_zeros=None,
         result = com._fill_zeros(result, x, y, name, fill_zeros)
         return result
 
-    def wrapper(left, right, name=name):
+    def wrapper(left, right, name=name, na_op=na_op):
 
         if isinstance(right, pd.DataFrame):
             return NotImplemented
 
-        time_converted = _TimeOp.maybe_convert_for_time_op(left, right, name)
+        time_converted = _TimeOp.maybe_convert_for_time_op(left, right, name, na_op)
 
         if time_converted is None:
             lvalues, rvalues = left, right
@@ -503,6 +587,7 @@ def _arith_method_SERIES(op, name, str_rep, fill_zeros=None,
             lvalues, rvalues = time_converted.lvalues, time_converted.rvalues
             dtype = time_converted.dtype
             wrap_results = time_converted.wrap_results
+            na_op = time_converted.na_op
 
         if isinstance(rvalues, pd.Series):
             rindex = getattr(rvalues,'index',rvalues)
@@ -527,8 +612,9 @@ def _arith_method_SERIES(op, name, str_rep, fill_zeros=None,
                                      name=name, dtype=dtype)
         else:
             # scalars
-            if hasattr(lvalues, 'values'):
+            if hasattr(lvalues, 'values') and not isinstance(lvalues, pd.DatetimeIndex):
                 lvalues = lvalues.values
+
             return left._constructor(wrap_results(na_op(lvalues, rvalues)),
                                      index=left.index, name=left.name,
                                      dtype=dtype)
@@ -544,17 +630,17 @@ def _comp_method_SERIES(op, name, str_rep, masker=False):
 
         # dispatch to the categorical if we have a categorical
         # in either operand
-        if com.is_categorical_dtype(x):
+        if is_categorical_dtype(x):
             return op(x,y)
-        elif com.is_categorical_dtype(y) and not lib.isscalar(y):
+        elif is_categorical_dtype(y) and not isscalar(y):
             return op(y,x)
 
-        if x.dtype == np.object_:
+        if is_object_dtype(x.dtype):
             if isinstance(y, list):
                 y = lib.list_to_object_array(y)
 
             if isinstance(y, (np.ndarray, pd.Series)):
-                if y.dtype != np.object_:
+                if not is_object_dtype(y.dtype):
                     result = lib.vec_compare(x, y.astype(np.object_), op)
                 else:
                     result = lib.vec_compare(x, y, op)
@@ -562,13 +648,44 @@ def _comp_method_SERIES(op, name, str_rep, masker=False):
                 result = lib.scalar_compare(x, y, op)
         else:
 
+            # we want to compare like types
+            # we only want to convert to integer like if
+            # we are not NotImplemented, otherwise
+            # we would allow datetime64 (but viewed as i8) against
+            # integer comparisons
+            if is_datetimelike_v_numeric(x, y):
+                raise TypeError("invalid type comparison")
+
+            # numpy does not like comparisons vs None
+            if isscalar(y) and isnull(y):
+                if name == '__ne__':
+                    return np.ones(len(x), dtype=bool)
+                else:
+                    return np.zeros(len(x), dtype=bool)
+
+            # we have a datetime/timedelta and may need to convert
+            mask = None
+            if needs_i8_conversion(x) or (not isscalar(y) and needs_i8_conversion(y)):
+
+                if isscalar(y):
+                    y = _index.convert_scalar(x,_values_from_object(y))
+                else:
+                    y = y.view('i8')
+
+                mask = isnull(x)
+
+                x = x.view('i8')
+
             try:
                 result = getattr(x, name)(y)
                 if result is NotImplemented:
                     raise TypeError("invalid type comparison")
-            except (AttributeError):
+            except AttributeError:
                 result = op(x, y)
 
+            if mask is not None and mask.any():
+                result[mask] = masker
+
         return result
 
     def wrapper(self, other, axis=None):
@@ -590,29 +707,24 @@ def _comp_method_SERIES(op, name, str_rep, masker=False):
             return self._constructor(na_op(self.values, np.asarray(other)),
                                      index=self.index).__finalize__(self)
         elif isinstance(other, pd.Categorical):
-            if not com.is_categorical_dtype(self):
+            if not is_categorical_dtype(self):
                 msg = "Cannot compare a Categorical for op {op} with Series of dtype {typ}.\n"\
                       "If you want to compare values, use 'series <op> np.asarray(other)'."
                 raise TypeError(msg.format(op=op,typ=self.dtype))
 
 
-        mask = isnull(self)
-
-        if com.is_categorical_dtype(self):
+        if is_categorical_dtype(self):
             # cats are a special case as get_values() would return an ndarray, which would then
             # not take categories ordering into account
             # we can go directly to op, as the na_op would just test again and dispatch to it.
             res = op(self.values, other)
         else:
             values = self.get_values()
-            other = _index.convert_scalar(values,_values_from_object(other))
+            if is_list_like(other):
+                other = np.asarray(other)
 
-            if issubclass(values.dtype.type, (np.datetime64, np.timedelta64)):
-                values = values.view('i8')
-
-            # scalars
             res = na_op(values, other)
-            if np.isscalar(res):
+            if isscalar(res):
                 raise TypeError('Could not compare %s type with Series'
                                 % type(other))
 
@@ -621,11 +733,6 @@ def _comp_method_SERIES(op, name, str_rep, masker=False):
 
         res = pd.Series(res, index=self.index, name=self.name,
                         dtype='bool')
-
-        # mask out the invalids
-        if mask.any():
-            res[mask] = masker
-
         return res
     return wrapper
 
@@ -643,8 +750,7 @@ def _bool_method_SERIES(op, name, str_rep):
                 y = lib.list_to_object_array(y)
 
             if isinstance(y, (np.ndarray, pd.Series)):
-                if (x.dtype == np.bool_ and
-                        y.dtype == np.bool_):  # pragma: no cover
+                if (is_bool_dtype(x.dtype) and is_bool_dtype(y.dtype)):
                     result = op(x, y)  # when would this be hit?
                 else:
                     x = com._ensure_object(x)
@@ -665,7 +771,7 @@ def _bool_method_SERIES(op, name, str_rep):
         return result
 
     def wrapper(self, other):
-        is_self_int_dtype = com.is_integer_dtype(self.dtype)
+        is_self_int_dtype = is_integer_dtype(self.dtype)
 
         fill_int = lambda x: x.fillna(0)
         fill_bool = lambda x: x.fillna(False).astype(bool)
@@ -673,7 +779,7 @@ def _bool_method_SERIES(op, name, str_rep):
         if isinstance(other, pd.Series):
             name = _maybe_match_name(self, other)
             other = other.reindex_like(self)
-            is_other_int_dtype = com.is_integer_dtype(other.dtype)
+            is_other_int_dtype = is_integer_dtype(other.dtype)
             other = fill_int(other) if is_other_int_dtype else fill_bool(other)
 
             filler = fill_int if is_self_int_dtype and is_other_int_dtype else fill_bool
@@ -686,7 +792,7 @@ def _bool_method_SERIES(op, name, str_rep):
 
         else:
             # scalars, list, tuple, np.array
-            filler = fill_int if is_self_int_dtype and com.is_integer_dtype(np.asarray(other)) else fill_bool
+            filler = fill_int if is_self_int_dtype and is_integer_dtype(np.asarray(other)) else fill_bool
             return filler(self._constructor(na_op(self.values, other),
                                     index=self.index)).__finalize__(self)
 
@@ -1046,7 +1152,7 @@ def _arith_method_PANEL(op, name, str_rep=None, fill_zeros=None,
 
     # work only for scalars
     def f(self, other):
-        if not np.isscalar(other):
+        if not isscalar(other):
             raise ValueError('Simple arithmetic with %s can only be '
                              'done with scalar values' %
                              self._constructor.__name__)
