@@ -14,7 +14,6 @@ from pandas.io.data import DataReader, SymbolWarning, RemoteDataError, _yahoo_co
 from pandas.util.testing import (assert_series_equal, assert_produces_warning,
                                  network, assert_frame_equal)
 import pandas.util.testing as tm
-from numpy.testing import assert_array_equal
 
 if compat.PY3:
     from urllib.error import HTTPError
@@ -28,12 +27,19 @@ def _skip_if_no_lxml():
     except ImportError:
         raise nose.SkipTest("no lxml")
 
+def _skip_if_no_bs():
+    try:
+        import bs4
+        import html5lib
+    except ImportError:
+        raise nose.SkipTest("no html5lib/bs4")
+
 
 def assert_n_failed_equals_n_null_columns(wngs, obj, cls=SymbolWarning):
     all_nan_cols = pd.Series(dict((k, pd.isnull(v).all()) for k, v in
                                   compat.iteritems(obj)))
     n_all_nan_cols = all_nan_cols.sum()
-    valid_warnings = pd.Series([wng for wng in wngs if isinstance(wng, cls)])
+    valid_warnings = pd.Series([wng for wng in wngs if wng.category == cls])
     assert_equal(len(valid_warnings), n_all_nan_cols)
     failed_symbols = all_nan_cols[all_nan_cols].index
     msgs = valid_warnings.map(lambda x: x.message)
@@ -79,18 +85,18 @@ class TestGoogle(tm.TestCase):
         for locale in self.locales:
             with tm.set_locale(locale):
                 df = web.get_data_google('GOOG').sort_index()
-            self.assertEqual(df.Volume.ix['OCT-08-2010'], 2863473)
+            self.assertEqual(df.Volume.ix['JAN-02-2015'], 1446662)
 
     @network
     def test_get_multi1(self):
         for locale in self.locales:
             sl = ['AAPL', 'AMZN', 'GOOG']
             with tm.set_locale(locale):
-                pan = web.get_data_google(sl, '2012')
-            ts = pan.Close.GOOG.index[pan.Close.AAPL > pan.Close.GOOG]
+                pan = web.get_data_google(sl, '2012', '2013')
+            ts = pan.Close.GOOG.index[pan.Close.AAPL < pan.Close.GOOG]
             if (hasattr(pan, 'Close') and hasattr(pan.Close, 'GOOG') and
                 hasattr(pan.Close, 'AAPL')):
-                self.assertEqual(ts[0].dayofyear, 96)
+                self.assertEqual(ts[0].dayofyear, 3)
             else:
                 self.assertRaises(AttributeError, lambda: pan.Close)
 
@@ -100,6 +106,11 @@ class TestGoogle(tm.TestCase):
         pan = web.get_data_google(sl, '2012')
         self.assertIn('INVALID', pan.minor_axis)
 
+    @network
+    def test_get_multi_all_invalid(self):
+        sl = ['INVALID', 'INVALID2', 'INVALID3']
+        self.assertRaises(RemoteDataError, web.get_data_google, sl, '2012')
+
     @network
     def test_get_multi2(self):
         with warnings.catch_warnings(record=True) as w:
@@ -112,11 +123,27 @@ class TestGoogle(tm.TestCase):
 
                 # sanity checking
 
-                assert np.issubdtype(result.dtype, np.floating)
+                self.assertTrue(np.issubdtype(result.dtype, np.floating))
                 result = pan.Open.ix['Jan-15-12':'Jan-20-12']
                 self.assertEqual((4, 3), result.shape)
                 assert_n_failed_equals_n_null_columns(w, result)
 
+    @network
+    def test_dtypes(self):
+        #GH3995, #GH8980
+        data = web.get_data_google('F', start='JAN-01-10', end='JAN-27-13')
+        self.assertTrue(np.issubdtype(data.Open.dtype, np.number))
+        self.assertTrue(np.issubdtype(data.Close.dtype, np.number))
+        self.assertTrue(np.issubdtype(data.Low.dtype, np.number))
+        self.assertTrue(np.issubdtype(data.High.dtype, np.number))
+        self.assertTrue(np.issubdtype(data.Volume.dtype, np.number))
+
+    @network
+    def test_unicode_date(self):
+        #GH8967
+        data = web.get_data_google('F', start='JAN-01-10', end='JAN-27-13')
+        self.assertEqual(data.index.name, 'Date')
+
 
 class TestYahoo(tm.TestCase):
     @classmethod
@@ -167,7 +194,7 @@ class TestYahoo(tm.TestCase):
         raise nose.SkipTest('unreliable test, receive partial components back for dow_jones')
 
         df = web.get_components_yahoo('^DJI') #Dow Jones
-        assert isinstance(df, pd.DataFrame)
+        self.assertIsInstance(df, pd.DataFrame)
         self.assertEqual(len(df), 30)
 
     @network
@@ -175,7 +202,7 @@ class TestYahoo(tm.TestCase):
         raise nose.SkipTest('unreliable test, receive partial components back for dax')
 
         df = web.get_components_yahoo('^GDAXI') #DAX
-        assert isinstance(df, pd.DataFrame)
+        self.assertIsInstance(df, pd.DataFrame)
         self.assertEqual(len(df), 30)
         self.assertEqual(df[df.name.str.contains('adidas', case=False)].index,
                          'ADS.DE')
@@ -186,13 +213,13 @@ class TestYahoo(tm.TestCase):
         raise nose.SkipTest('unreliable test, receive partial components back for nasdaq_100')
 
         df = web.get_components_yahoo('^NDX') #NASDAQ-100
-        assert isinstance(df, pd.DataFrame)
+        self.assertIsInstance(df, pd.DataFrame)
 
         if len(df) > 1:
             # Usual culprits, should be around for a while
-            assert 'AAPL' in df.index
-            assert 'GOOG' in df.index
-            assert 'AMZN' in df.index
+            self.assertTrue('AAPL' in df.index)
+            self.assertTrue('GOOG' in df.index)
+            self.assertTrue('AMZN' in df.index)
         else:
             expected = DataFrame({'exchange': 'N/A', 'name': '@^NDX'},
                                  index=['@^NDX'])
@@ -205,6 +232,27 @@ class TestYahoo(tm.TestCase):
         # just test that we succeed
         web.get_data_yahoo('GOOG')
 
+    @network
+    def test_get_data_interval(self):
+        # daily interval data
+        pan = web.get_data_yahoo('XOM', '2013-01-01', '2013-12-31', interval='d')
+        self.assertEqual(len(pan), 252)
+
+        # weekly interval data
+        pan = web.get_data_yahoo('XOM', '2013-01-01', '2013-12-31', interval='w')
+        self.assertEqual(len(pan), 53)
+
+        # montly interval data
+        pan = web.get_data_yahoo('XOM', '2013-01-01', '2013-12-31', interval='m')
+        self.assertEqual(len(pan), 12)
+
+        # dividend data
+        pan = web.get_data_yahoo('XOM', '2013-01-01', '2013-12-31', interval='v')
+        self.assertEqual(len(pan), 4)
+
+        # test fail on invalid interval
+        self.assertRaises(ValueError, web.get_data_yahoo, 'XOM', interval='NOT VALID')
+
     @network
     def test_get_data_multiple_symbols(self):
         # just test that we succeed
@@ -219,7 +267,7 @@ class TestYahoo(tm.TestCase):
         self.assertEqual(len(result), 3)
 
         # sanity checking
-        assert np.issubdtype(result.dtype, np.floating)
+        self.assertTrue(np.issubdtype(result.dtype, np.floating))
 
         expected = np.array([[18.99,  28.4, 25.18],
                              [18.58, 28.31, 25.13],
@@ -239,7 +287,7 @@ class TestYahoo(tm.TestCase):
             self.assertEqual(result, 1.0)
 
         # sanity checking
-        assert np.issubdtype(pan.values.dtype, np.floating)
+        self.assertTrue(np.issubdtype(pan.values.dtype, np.floating))
 
 
 class TestYahooOptions(tm.TestCase):
@@ -247,19 +295,19 @@ class TestYahooOptions(tm.TestCase):
     def setUpClass(cls):
         super(TestYahooOptions, cls).setUpClass()
         _skip_if_no_lxml()
+        _skip_if_no_bs()
 
         # aapl has monthlies
         cls.aapl = web.Options('aapl', 'yahoo')
-        today = datetime.today()
-        cls.year = today.year
-        cls.month = today.month + 1
-        if cls.month > 12:
-            cls.year = cls.year + 1
-            cls.month = 1
-        cls.expiry = datetime(cls.year, cls.month, 1)
+        d = (Timestamp.today() + pd.offsets.MonthBegin(1)).normalize()
+        cls.year = d.year
+        cls.month = d.month
+        cls.expiry = d
+        cls.expiry2 = d + pd.offsets.MonthBegin(1)
         cls.dirpath = tm.get_data_path()
         cls.html1 = os.path.join(cls.dirpath, 'yahoo_options1.html')
         cls.html2 = os.path.join(cls.dirpath, 'yahoo_options2.html')
+        cls.html3 = os.path.join(cls.dirpath, 'yahoo_options3.html') #Empty table GH#22
         cls.data1 = cls.aapl._option_frames_from_url(cls.html1)['puts']
 
     @classmethod
@@ -283,7 +331,7 @@ class TestYahooOptions(tm.TestCase):
     def test_get_near_stock_price(self):
         try:
             options = self.aapl.get_near_stock_price(call=True, put=True,
-                                                     expiry=self.expiry)
+                                                     expiry=[self.expiry,self.expiry2])
         except RemoteDataError as e:
             raise nose.SkipTest(e)
         self.assertTrue(len(options) > 1)
@@ -337,9 +385,20 @@ class TestYahooOptions(tm.TestCase):
         self.assertTrue(len(data) > 1)
 
     @network
+    def test_get_underlying_price(self):
+        #GH7
+        try:
+            options_object = web.Options('^spxpm', 'yahoo')
+            url = options_object._yahoo_url_from_expiry(options_object.expiry_dates[0])
+            root = options_object._parse_url(url)
+            quote_price = options_object._underlying_price_from_root(root)
+        except RemoteDataError as e:
+            raise nose.SkipTest(e)
+        self.assertIsInstance(quote_price, float)
+
     def test_sample_page_price_quote_time1(self):
         #Tests the weekend quote time format
-        price, quote_time = self.aapl._get_underlying_price(self.html1)
+        price, quote_time = self.aapl._underlying_price_and_time_from_url(self.html1)
         self.assertIsInstance(price, (int, float, complex))
         self.assertIsInstance(quote_time, (datetime, Timestamp))
 
@@ -362,7 +421,7 @@ class TestYahooOptions(tm.TestCase):
     def test_sample_page_price_quote_time2(self):
         #Tests the EDT page format
         #regression test for #8741
-        price, quote_time = self.aapl._get_underlying_price(self.html2)
+        price, quote_time = self.aapl._underlying_price_and_time_from_url(self.html2)
         self.assertIsInstance(price, (int, float, complex))
         self.assertIsInstance(quote_time, (datetime, Timestamp))
 
@@ -380,6 +439,12 @@ class TestYahooOptions(tm.TestCase):
 
         self.assertTrue(len(data) > 1)
 
+    @network
+    def test_empty_table(self):
+        #GH22
+        empty = self.aapl._option_frames_from_url(self.html3)['puts']
+        self.assertTrue(len(empty) == 0)
+
 
 class TestOptionsWarnings(tm.TestCase):
     @classmethod
@@ -404,17 +469,17 @@ class TestDataReader(tm.TestCase):
     @network
     def test_read_yahoo(self):
         gs = DataReader("GS", "yahoo")
-        assert isinstance(gs, DataFrame)
+        self.assertIsInstance(gs, DataFrame)
 
     @network
     def test_read_google(self):
         gs = DataReader("GS", "google")
-        assert isinstance(gs, DataFrame)
+        self.assertIsInstance(gs, DataFrame)
 
     @network
     def test_read_fred(self):
         vix = DataReader("VIXCLS", "fred")
-        assert isinstance(vix, DataFrame)
+        self.assertIsInstance(vix, DataFrame)
 
     @network
     def test_read_famafrench(self):
@@ -422,8 +487,8 @@ class TestDataReader(tm.TestCase):
                      "F-F_Research_Data_Factors_weekly", "6_Portfolios_2x3",
                      "F-F_ST_Reversal_Factor", "F-F_Momentum_Factor"):
             ff = DataReader(name, "famafrench")
-            assert ff
-            assert isinstance(ff, dict)
+            self.assertTrue(ff is not None)
+            self.assertIsInstance(ff, dict)
 
 
 class TestFred(tm.TestCase):
@@ -437,10 +502,7 @@ class TestFred(tm.TestCase):
         end = datetime(2013, 1, 27)
 
         received = web.DataReader("GDP", "fred", start, end)['GDP'].tail(1)[0]
-
-        # < 7/30/14 16535 was returned
-        #self.assertEqual(int(received), 16535)
-        self.assertEqual(int(received), 16502)
+        self.assertTrue(int(received) > 10000)
 
         self.assertRaises(Exception, web.DataReader, "NON EXISTENT SERIES",
                           'fred', start, end)
@@ -450,7 +512,7 @@ class TestFred(tm.TestCase):
         start = datetime(2010, 1, 1)
         end = datetime(2013, 1, 27)
         df = web.DataReader("DFII5", "fred", start, end)
-        assert pd.isnull(df.ix['2010-01-01'][0])
+        self.assertTrue(pd.isnull(df.ix['2010-01-01'][0]))
 
     @network
     def test_fred_parts(self):
@@ -462,7 +524,7 @@ class TestFred(tm.TestCase):
         self.assertEqual(df.ix['2010-05-01'][0], 217.23)
 
         t = df.CPIAUCSL.values
-        assert np.issubdtype(t.dtype, np.floating)
+        self.assertTrue(np.issubdtype(t.dtype, np.floating))
         self.assertEqual(t.shape, (37,))
 
     @network
@@ -473,7 +535,7 @@ class TestFred(tm.TestCase):
                     [848.3],
                     [933.3]]
         result = web.get_data_fred("A09024USA144NNBR", start="1915").ix[:5]
-        assert_array_equal(result.values, np.array(expected))
+        tm.assert_numpy_array_equal(result.values, np.array(expected))
 
     @network
     def test_invalid_series(self):
