@@ -1,14 +1,10 @@
 # pylint: disable=E1101,E1103,W0232
-import operator
-
-from datetime import datetime, date, timedelta
+from datetime import datetime, timedelta
 import numpy as np
-from pandas.core.base import PandasObject
-
 import pandas.tseries.frequencies as frequencies
 from pandas.tseries.frequencies import get_freq_code as _gfc
 from pandas.tseries.index import DatetimeIndex, Int64Index, Index
-from pandas.tseries.base import DatetimeIndexOpsMixin
+from pandas.tseries.base import DatelikeOps, DatetimeIndexOpsMixin
 from pandas.tseries.tools import parse_time_string
 import pandas.tseries.offsets as offsets
 
@@ -23,8 +19,11 @@ from pandas._period import (
 import pandas.core.common as com
 from pandas.core.common import (isnull, _INT64_DTYPE, _maybe_box,
                                 _values_from_object, ABCSeries,
-                                is_integer, is_float)
+                                is_integer, is_float, is_object_dtype,
+                                is_float_dtype)
 from pandas import compat
+from pandas.util.decorators import cache_readonly
+
 from pandas.lib import Timestamp, Timedelta
 import pandas.lib as lib
 import pandas.tslib as tslib
@@ -58,6 +57,8 @@ def dt64arr_to_periodarr(data, freq, tz):
 
 # --- Period index sketch
 
+_DIFFERENT_FREQ_ERROR = "Input has different freq={1} from PeriodIndex(freq={0})"
+
 def _period_index_cmp(opname, nat_result=False):
     """
     Wrap comparison operations to convert datetime-like to datetime64
@@ -65,13 +66,16 @@ def _period_index_cmp(opname, nat_result=False):
     def wrapper(self, other):
         if isinstance(other, Period):
             func = getattr(self.values, opname)
+            other_base, _ = _gfc(other.freq)
             if other.freq != self.freq:
-                raise AssertionError("Frequencies must be equal")
+                msg = _DIFFERENT_FREQ_ERROR.format(self.freqstr, other.freqstr)
+                raise ValueError(msg)
 
             result = func(other.ordinal)
         elif isinstance(other, PeriodIndex):
             if other.freq != self.freq:
-                raise AssertionError("Frequencies must be equal")
+                msg = _DIFFERENT_FREQ_ERROR.format(self.freqstr, other.freqstr)
+                raise ValueError(msg)
 
             result = getattr(self.values, opname)(other.values)
 
@@ -96,7 +100,7 @@ def _period_index_cmp(opname, nat_result=False):
     return wrapper
 
 
-class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
+class PeriodIndex(DatelikeOps, DatetimeIndexOpsMixin, Int64Index):
     """
     Immutable ndarray holding ordinal values indicating regular periods in
     time such as particular years, quarters, months, etc. A value of 1 is the
@@ -114,20 +118,20 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
 
     Parameters
     ----------
-    data  : array-like (1-dimensional), optional
+    data : array-like (1-dimensional), optional
         Optional period-like data to construct index with
     dtype : NumPy dtype (default: i8)
-    copy  : bool
+    copy : bool
         Make a copy of input ndarray
     freq : string or period object, optional
         One of pandas period strings or corresponding objects
     start : starting value, period-like, optional
         If data is None, used as the start point in generating regular
         period data.
-    periods  : int, optional, > 0
+    periods : int, optional, > 0
         Number of periods to generate, if generating index. Takes precedence
         over end argument
-    end   : end value, period-like, optional
+    end : end value, period-like, optional
         If periods is none, generated index will extend to first conforming
         period on or just past end argument
     year : int, array, or Series, default None
@@ -164,8 +168,6 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
     def __new__(cls, data=None, ordinal=None, freq=None, start=None, end=None,
                 periods=None, copy=False, name=None, tz=None, **kwargs):
 
-        freq = frequencies.get_standard_freq(freq)
-
         if periods is not None:
             if is_float(periods):
                 periods = int(periods)
@@ -239,8 +241,8 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
                 else:
                     base1, _ = _gfc(data.freq)
                     base2, _ = _gfc(freq)
-                    data = period.period_asfreq_arr(data.values, base1,
-                                                   base2, 1)
+                    data = period.period_asfreq_arr(data.values,
+                                                    base1, base2, 1)
             else:
                 if freq is None and len(data) > 0:
                     freq = getattr(data[0], 'freq', None)
@@ -263,13 +265,34 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
 
     @classmethod
     def _simple_new(cls, values, name=None, freq=None, **kwargs):
+        if not getattr(values,'dtype',None):
+            values = np.array(values,copy=False)
+        if is_object_dtype(values):
+            return PeriodIndex(values, name=name, freq=freq, **kwargs)
+
         result = object.__new__(cls)
         result._data = values
         result.name = name
-        result.freq = freq
+        if freq is None:
+            raise ValueError('freq is not specified')
+        result.freq = Period._maybe_convert_freq(freq)
         result._reset_identity()
         return result
 
+    def _shallow_copy(self, values=None, infer=False, **kwargs):
+        """ we always want to return a PeriodIndex """
+        return super(PeriodIndex, self)._shallow_copy(values=values, infer=False, **kwargs)
+
+    def _coerce_scalar_to_index(self, item):
+        """
+        we need to coerce a scalar to a compat for our index type
+
+        Parameters
+        ----------
+        item : scalar item to coerce
+        """
+        return PeriodIndex([item], **self._get_attributes_dict())
+
     @property
     def _na_value(self):
         return self._box_func(tslib.iNaT)
@@ -285,6 +308,30 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
             return False
         return key.ordinal in self._engine
 
+    def __array_wrap__(self, result, context=None):
+        """
+        Gets called after a ufunc. Needs additional handling as
+        PeriodIndex stores internal data as int dtype
+
+        Replace this to __numpy_ufunc__ in future version
+        """
+        if isinstance(context, tuple) and len(context) > 0:
+            func = context[0]
+            if (func is np.add):
+                return self._add_delta(context[1][1])
+            elif (func is np.subtract):
+                return self._add_delta(-context[1][1])
+            elif isinstance(func, np.ufunc):
+                if 'M->M' not in func.types:
+                    msg = "ufunc '{0}' not supported for the PeriodIndex"
+                    # This should be TypeError, but TypeError cannot be raised
+                    # from here because numpy catches.
+                    raise ValueError(msg.format(func.__name__))
+
+        if com.is_bool_dtype(result):
+            return result
+        return PeriodIndex(result, freq=self.freq, name=self.name)
+
     @property
     def _box_func(self):
         return lambda x: Period._from_ordinal(ordinal=x, freq=self.freq)
@@ -331,7 +378,8 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
     def searchsorted(self, key, side='left'):
         if isinstance(key, Period):
             if key.freq != self.freq:
-                raise ValueError("Different period frequency: %s" % key.freq)
+                msg = _DIFFERENT_FREQ_ERROR.format(self.freqstr, key.freqstr)
+                raise ValueError(msg)
             key = key.ordinal
         elif isinstance(key, compat.string_types):
             key = Period(key, freq=self.freq).ordinal
@@ -354,10 +402,6 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         values = self.values
         return ((values[1:] - values[:-1]) < 2).all()
 
-    @property
-    def freqstr(self):
-        return self.freq
-
     def asfreq(self, freq=None, how='E'):
         """
         Convert the PeriodIndex to the specified frequency `freq`.
@@ -404,11 +448,20 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         base1, mult1 = _gfc(self.freq)
         base2, mult2 = _gfc(freq)
 
-        if mult2 != 1:
-            raise ValueError('Only mult == 1 supported')
-
+        asi8 = self.asi8
+        # mult1 can't be negative or 0
         end = how == 'E'
-        new_data = period.period_asfreq_arr(self.values, base1, base2, end)
+        if end:
+            ordinal = asi8 + mult1 - 1
+        else:
+            ordinal = asi8
+
+        new_data = period.period_asfreq_arr(ordinal, base1, base2, end)
+
+        if self.hasnans:
+            mask = asi8 == tslib.iNaT
+            new_data[mask] = tslib.iNaT
+
         return self._simple_new(new_data, self.name, freq=freq)
 
     def to_datetime(self, dayfirst=False):
@@ -481,21 +534,35 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         new_data = period.periodarr_to_dt64arr(new_data.values, base)
         return DatetimeIndex(new_data, freq='infer', name=self.name)
 
-    def _add_delta(self, other):
+    def _maybe_convert_timedelta(self, other):
         if isinstance(other, (timedelta, np.timedelta64, offsets.Tick, Timedelta)):
-            offset = frequencies.to_offset(self.freq)
+            offset = frequencies.to_offset(self.freq.rule_code)
             if isinstance(offset, offsets.Tick):
                 nanos = tslib._delta_to_nanoseconds(other)
                 offset_nanos = tslib._delta_to_nanoseconds(offset)
                 if nanos % offset_nanos == 0:
-                    return self.shift(nanos // offset_nanos)
+                    return nanos // offset_nanos
         elif isinstance(other, offsets.DateOffset):
             freqstr = frequencies.get_standard_freq(other)
             base = frequencies.get_base_alias(freqstr)
+            if base == self.freq.rule_code:
+                return other.n
+        elif isinstance(other, np.ndarray):
+            if com.is_integer_dtype(other):
+                return other
+            elif com.is_timedelta64_dtype(other):
+                offset = frequencies.to_offset(self.freq)
+                if isinstance(offset, offsets.Tick):
+                    nanos = tslib._delta_to_nanoseconds(other)
+                    offset_nanos = tslib._delta_to_nanoseconds(offset)
+                    if (nanos % offset_nanos).all() == 0:
+                        return nanos // offset_nanos
+        msg = "Input has different freq from PeriodIndex(freq={0})"
+        raise ValueError(msg.format(self.freqstr))
 
-            if base == self.freq:
-                return self.shift(other.n)
-        raise ValueError("Input has different freq from PeriodIndex(freq={0})".format(self.freq))
+    def _add_delta(self, other):
+        ordinal_delta = self._maybe_convert_timedelta(other)
+        return self.shift(ordinal_delta)
 
     def shift(self, n):
         """
@@ -505,17 +572,21 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         ----------
         n : int
             Periods to shift by
-        freq : freq string
 
         Returns
         -------
         shifted : PeriodIndex
         """
         mask = self.values == tslib.iNaT
-        values = self.values + n
+        values = self.values + n * self.freq.n
         values[mask] = tslib.iNaT
         return PeriodIndex(data=values, name=self.name, freq=self.freq)
 
+    @cache_readonly
+    def dtype_str(self):
+        """ return the dtype str of the underlying data """
+        return self.inferred_type
+
     @property
     def inferred_type(self):
         # b/c data is represented as ints make sure we can't have ambiguous
@@ -533,8 +604,8 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         except (KeyError, IndexError):
             try:
                 asdt, parsed, reso = parse_time_string(key, self.freq)
-                grp = frequencies._infer_period_group(reso)
-                freqn = frequencies._period_group(self.freq)
+                grp = frequencies.Resolution.get_freq_group(reso)
+                freqn = frequencies.get_freq_group(self.freq)
 
                 vals = self.values
 
@@ -561,13 +632,13 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
             key = Period(key, self.freq).ordinal
             return _maybe_box(self, self._engine.get_value(s, key), series, key)
 
-    def get_indexer(self, target, method=None, limit=None):
+    def get_indexer(self, target, method=None, limit=None, tolerance=None):
         if hasattr(target, 'freq') and target.freq != self.freq:
             raise ValueError('target and index have different freq: '
                              '(%s, %s)' % (target.freq, self.freq))
-        return Index.get_indexer(self, target, method, limit)
+        return Index.get_indexer(self, target, method, limit, tolerance)
 
-    def get_loc(self, key, method=None):
+    def get_loc(self, key, method=None, tolerance=None):
         """
         Get integer location for requested label
 
@@ -587,9 +658,9 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
             except TypeError:
                 pass
 
-            key = Period(key, self.freq)
+            key = Period(key, freq=self.freq)
             try:
-                return Index.get_loc(self, key.ordinal, method=method)
+                return Index.get_loc(self, key.ordinal, method, tolerance)
             except KeyError:
                 raise KeyError(key)
 
@@ -659,9 +730,8 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
                              'ordered time series')
 
         key, parsed, reso = parse_time_string(key, self.freq)
-
-        grp = frequencies._infer_period_group(reso)
-        freqn = frequencies._period_group(self.freq)
+        grp = frequencies.Resolution.get_freq_group(reso)
+        freqn = frequencies.get_freq_group(self.freq)
         if reso in ['day', 'hour', 'minute', 'second'] and not grp < freqn:
             raise KeyError(key)
 
@@ -669,6 +739,10 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         return slice(self.searchsorted(t1.ordinal, side='left'),
                      self.searchsorted(t2.ordinal, side='right'))
 
+    def _convert_tolerance(self, tolerance):
+        tolerance = DatetimeIndexOpsMixin._convert_tolerance(self, tolerance)
+        return self._maybe_convert_timedelta(tolerance)
+
     def join(self, other, how='left', level=None, return_indexers=False):
         """
         See Index.join
@@ -684,12 +758,14 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         return self._apply_meta(result)
 
     def _assert_can_do_setop(self, other):
+        super(PeriodIndex, self)._assert_can_do_setop(other)
+
         if not isinstance(other, PeriodIndex):
             raise ValueError('can only call with other PeriodIndex-ed objects')
 
         if self.freq != other.freq:
-            raise ValueError('Only like-indexed PeriodIndexes compatible '
-                             'for join (for now)')
+            msg = _DIFFERENT_FREQ_ERROR.format(self.freqstr, other.freqstr)
+            raise ValueError(msg)
 
     def _wrap_union_result(self, other, result):
         name = self.name if self.name == other.name else None
@@ -721,30 +797,26 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
 
             return PeriodIndex(result, name=self.name, freq=self.freq)
 
-    def _format_native_types(self, na_rep=u('NaT'), **kwargs):
+    def _format_native_types(self, na_rep=u('NaT'), date_format=None, **kwargs):
 
         values = np.array(list(self), dtype=object)
         mask = isnull(self.values)
         values[mask] = na_rep
-
         imask = ~mask
-        values[imask] = np.array([u('%s') % dt for dt in values[imask]])
+
+        if date_format:
+            formatter = lambda dt: dt.strftime(date_format)
+        else:
+            formatter = lambda dt: u('%s') % dt
+        values[imask] = np.array([formatter(dt) for dt in values[imask]])
         return values
 
-    def __array_finalize__(self, obj):
-        if not self.ndim:  # pragma: no cover
-            return self.item()
-
-        self.freq = getattr(obj, 'freq', None)
-        self.name = getattr(obj, 'name', None)
-        self._reset_identity()
-
-    def take(self, indices, axis=None):
+    def take(self, indices, axis=0):
         """
         Analogous to ndarray.take
         """
         indices = com._ensure_platform_int(indices)
-        taken = self.values.take(indices, axis=axis)
+        taken = self.asi8.take(indices, axis=axis)
         return self._simple_new(taken, self.name, freq=self.freq)
 
     def append(self, other):
@@ -786,6 +858,17 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
                      for x in to_concat]
         return Index(com._concat_compat(to_concat), name=name)
 
+    def repeat(self, n):
+        """
+        Return a new Index of the values repeated n times.
+
+        See also
+        --------
+        numpy.ndarray.repeat
+        """
+        # overwrites method from DatetimeIndexOpsMixin
+        return self._shallow_copy(self.values.repeat(n))
+
     def __setstate__(self, state):
         """Necessary for making this object picklable"""
 
@@ -800,10 +883,8 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
                 data = np.empty(nd_state[1], dtype=nd_state[2])
                 np.ndarray.__setstate__(data, nd_state)
 
-                try:  # backcompat
-                    self.freq = own_state[1]
-                except:
-                    pass
+                # backcompat
+                self.freq = Period._maybe_convert_freq(own_state[1])
 
             else:  # pragma: no cover
                 data = np.empty(state)
@@ -813,6 +894,7 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
 
         else:
             raise Exception("invalid pickle state")
+
     _unpickle_compat = __setstate__
 
     def tz_convert(self, tz):
@@ -866,10 +948,13 @@ PeriodIndex._add_logical_methods_disabled()
 PeriodIndex._add_datetimelike_methods()
 
 
-def _get_ordinal_range(start, end, periods, freq):
+def _get_ordinal_range(start, end, periods, freq, mult=1):
     if com._count_not_none(start, end, periods) < 2:
         raise ValueError('Must specify 2 of start, end, periods')
 
+    if freq is not None:
+        _, mult = _gfc(freq)
+
     if start is not None:
         start = Period(start, freq)
     if end is not None:
@@ -893,15 +978,16 @@ def _get_ordinal_range(start, end, periods, freq):
             raise ValueError('Could not infer freq from start/end')
 
     if periods is not None:
+        periods = periods * mult
         if start is None:
-            data = np.arange(end.ordinal - periods + 1,
-                             end.ordinal + 1,
+            data = np.arange(end.ordinal - periods + mult,
+                             end.ordinal + 1, mult,
                              dtype=np.int64)
         else:
-            data = np.arange(start.ordinal, start.ordinal + periods,
+            data = np.arange(start.ordinal, start.ordinal + periods, mult,
                              dtype=np.int64)
     else:
-        data = np.arange(start.ordinal, end.ordinal + 1, dtype=np.int64)
+        data = np.arange(start.ordinal, end.ordinal + 1, mult, dtype=np.int64)
 
     return data, freq
 
@@ -925,8 +1011,6 @@ def _range_from_fields(year=None, month=None, quarter=None, day=None,
             base = frequencies.FreqGroup.FR_QTR
         else:
             base, mult = _gfc(freq)
-            if mult != 1:
-                raise ValueError('Only mult == 1 supported')
             if base != frequencies.FreqGroup.FR_QTR:
                 raise AssertionError("base must equal FR_QTR")
 
@@ -937,9 +1021,6 @@ def _range_from_fields(year=None, month=None, quarter=None, day=None,
             ordinals.append(val)
     else:
         base, mult = _gfc(freq)
-        if mult != 1:
-            raise ValueError('Only mult == 1 supported')
-
         arrays = _make_field_arrays(year, month, day, hour, minute, second)
         for y, mth, d, h, mn, s in zip(*arrays):
             ordinals.append(period.period_ordinal(y, mth, d, h, mn, s, 0, 0, base))
@@ -974,8 +1055,8 @@ def period_range(start=None, end=None, periods=None, freq='D', name=None):
 
     Parameters
     ----------
-    start :
-    end :
+    start : starting value, period-like, optional
+    end : ending value, period-like, optional
     periods : int, default None
         Number of periods in the index
     freq : str/DateOffset, default 'D'
