@@ -4,10 +4,11 @@ from datetime import datetime, timedelta
 import numpy as np
 import pandas as pd
 from pandas.tseries.base import DatetimeIndexOpsMixin
-from pandas.util.testing import assertRaisesRegexp, assert_isinstance
+from pandas.util.testing import assertRaisesRegexp, assertIsInstance
 from pandas.tseries.common import is_datetimelike
 from pandas import (Series, Index, Int64Index, Timestamp, DatetimeIndex, PeriodIndex,
                     TimedeltaIndex, Timedelta, timedelta_range, date_range, Float64Index)
+import pandas.tseries.offsets as offsets
 import pandas.tslib as tslib
 import nose
 
@@ -110,6 +111,35 @@ class TestDatetimeIndexOps(Ops):
             self.assertTrue(pd.isnull(getattr(obj, op)()))
 
     def test_representation(self):
+
+        idx = []
+        idx.append(DatetimeIndex([], freq='D'))
+        idx.append(DatetimeIndex(['2011-01-01'], freq='D'))
+        idx.append(DatetimeIndex(['2011-01-01', '2011-01-02'], freq='D'))
+        idx.append(DatetimeIndex(['2011-01-01', '2011-01-02', '2011-01-03'], freq='D'))
+        idx.append(DatetimeIndex(['2011-01-01 09:00', '2011-01-01 10:00', '2011-01-01 11:00'],
+                                 freq='H', tz='Asia/Tokyo'))
+        idx.append(DatetimeIndex(['2011-01-01 09:00', '2011-01-01 10:00', pd.NaT],
+                                 tz='US/Eastern'))
+        idx.append(DatetimeIndex(['2011-01-01 09:00', '2011-01-01 10:00', pd.NaT],
+                                 tz='UTC'))
+
+        exp = []
+        exp.append("""DatetimeIndex([], dtype='datetime64[ns]', freq='D')""")
+        exp.append("""DatetimeIndex(['2011-01-01'], dtype='datetime64[ns]', freq='D')""")
+        exp.append("""DatetimeIndex(['2011-01-01', '2011-01-02'], dtype='datetime64[ns]', freq='D')""")
+        exp.append("""DatetimeIndex(['2011-01-01', '2011-01-02', '2011-01-03'], dtype='datetime64[ns]', freq='D')""")
+        exp.append("""DatetimeIndex(['2011-01-01 09:00:00+09:00', '2011-01-01 10:00:00+09:00', '2011-01-01 11:00:00+09:00'], dtype='datetime64[ns, Asia/Tokyo]', freq='H')""")
+        exp.append("""DatetimeIndex(['2011-01-01 09:00:00-05:00', '2011-01-01 10:00:00-05:00', 'NaT'], dtype='datetime64[ns, US/Eastern]', freq=None)""")
+        exp.append("""DatetimeIndex(['2011-01-01 09:00:00+00:00', '2011-01-01 10:00:00+00:00', 'NaT'], dtype='datetime64[ns, UTC]', freq=None)""")
+
+        with pd.option_context('display.width', 300):
+            for indx, expected in zip(idx, exp):
+                for func in ['__repr__', '__unicode__', '__str__']:
+                    result = getattr(indx, func)()
+                    self.assertEqual(result, expected)
+
+    def test_representation_to_series(self):
         idx1 = DatetimeIndex([], freq='D')
         idx2 = DatetimeIndex(['2011-01-01'], freq='D')
         idx3 = DatetimeIndex(['2011-01-01', '2011-01-02'], freq='D')
@@ -118,25 +148,41 @@ class TestDatetimeIndexOps(Ops):
                              freq='H', tz='Asia/Tokyo')
         idx6 = DatetimeIndex(['2011-01-01 09:00', '2011-01-01 10:00', pd.NaT],
                              tz='US/Eastern')
+        idx7 = DatetimeIndex(['2011-01-01 09:00', '2011-01-02 10:15'])
 
-        exp1 = """DatetimeIndex([], dtype='datetime64[ns]', freq='D', tz=None)"""
+        exp1 = """Series([], dtype: datetime64[ns])"""
 
-        exp2 = """DatetimeIndex(['2011-01-01'], dtype='datetime64[ns]', freq='D', tz=None)"""
+        exp2 = """0   2011-01-01
+dtype: datetime64[ns]"""
 
-        exp3 = """DatetimeIndex(['2011-01-01', '2011-01-02'], dtype='datetime64[ns]', freq='D', tz=None)"""
+        exp3 = """0   2011-01-01
+1   2011-01-02
+dtype: datetime64[ns]"""
 
-        exp4 = """DatetimeIndex(['2011-01-01', '2011-01-02', '2011-01-03'], dtype='datetime64[ns]', freq='D', tz=None)"""
+        exp4 = """0   2011-01-01
+1   2011-01-02
+2   2011-01-03
+dtype: datetime64[ns]"""
 
-        exp5 = """DatetimeIndex(['2011-01-01 09:00:00+09:00', '2011-01-01 10:00:00+09:00', '2011-01-01 11:00:00+09:00'], dtype='datetime64[ns]', freq='H', tz='Asia/Tokyo')"""
+        exp5 = """0   2011-01-01 09:00:00+09:00
+1   2011-01-01 10:00:00+09:00
+2   2011-01-01 11:00:00+09:00
+dtype: datetime64[ns, Asia/Tokyo]"""
 
-        exp6 = """DatetimeIndex(['2011-01-01 09:00:00-05:00', '2011-01-01 10:00:00-05:00', 'NaT'], dtype='datetime64[ns]', freq=None, tz='US/Eastern')"""
+        exp6 = """0   2011-01-01 09:00:00-05:00
+1   2011-01-01 10:00:00-05:00
+2                         NaT
+dtype: datetime64[ns, US/Eastern]"""
+
+        exp7 = """0   2011-01-01 09:00:00
+1   2011-01-02 10:15:00
+dtype: datetime64[ns]"""
 
         with pd.option_context('display.width', 300):
-            for idx, expected in zip([idx1, idx2, idx3, idx4, idx5, idx6],
-                                     [exp1, exp2, exp3, exp4, exp5, exp6]):
-                for func in ['__repr__', '__unicode__', '__str__']:
-                    result = getattr(idx, func)()
-                    self.assertEqual(result, expected)
+            for idx, expected in zip([idx1, idx2, idx3, idx4, idx5, idx6, idx7],
+                                     [exp1, exp2, exp3, exp4, exp5, exp6, exp7]):
+                result = repr(Series(idx))
+                self.assertEqual(result, expected)
 
     def test_summary(self):
         # GH9116
@@ -297,6 +343,72 @@ Freq: H"""
                                        ['2015', '2015', '2016'], ['2015', '2015', '2014'])):
             tm.assertIn(idx[0], idx)
 
+    def test_order(self):
+        # with freq
+        idx1 = DatetimeIndex(['2011-01-01', '2011-01-02', '2011-01-03'], freq='D', name='idx')
+        idx2 = DatetimeIndex(['2011-01-01 09:00', '2011-01-01 10:00', '2011-01-01 11:00'],
+                             freq='H', tz='Asia/Tokyo', name='tzidx')
+
+        for idx in [idx1, idx2]:
+            ordered = idx.sort_values()
+            self.assert_index_equal(ordered, idx)
+            self.assertEqual(ordered.freq, idx.freq)
+
+            ordered = idx.sort_values(ascending=False)
+            expected = idx[::-1]
+            self.assert_index_equal(ordered, expected)
+            self.assertEqual(ordered.freq, expected.freq)
+            self.assertEqual(ordered.freq.n, -1)
+
+            ordered, indexer = idx.sort_values(return_indexer=True)
+            self.assert_index_equal(ordered, idx)
+            self.assert_numpy_array_equal(indexer, np.array([0, 1, 2]))
+            self.assertEqual(ordered.freq, idx.freq)
+
+            ordered, indexer = idx.sort_values(return_indexer=True, ascending=False)
+            expected = idx[::-1]
+            self.assert_index_equal(ordered, expected)
+            self.assert_numpy_array_equal(indexer, np.array([2, 1, 0]))
+            self.assertEqual(ordered.freq, expected.freq)
+            self.assertEqual(ordered.freq.n, -1)
+
+        # without freq
+        idx1 = DatetimeIndex(['2011-01-01', '2011-01-03', '2011-01-05',
+                              '2011-01-02', '2011-01-01'], name='idx1')
+        exp1 = DatetimeIndex(['2011-01-01', '2011-01-01', '2011-01-02',
+                              '2011-01-03', '2011-01-05'], name='idx1')
+
+        idx2 = DatetimeIndex(['2011-01-01', '2011-01-03', '2011-01-05',
+                              '2011-01-02', '2011-01-01'],
+                             tz='Asia/Tokyo', name='idx2')
+        exp2 = DatetimeIndex(['2011-01-01', '2011-01-01', '2011-01-02',
+                              '2011-01-03', '2011-01-05'],
+                             tz='Asia/Tokyo', name='idx2')
+
+        idx3 = DatetimeIndex([pd.NaT, '2011-01-03', '2011-01-05',
+                              '2011-01-02', pd.NaT], name='idx3')
+        exp3 = DatetimeIndex([pd.NaT, pd.NaT, '2011-01-02', '2011-01-03',
+                              '2011-01-05'], name='idx3')
+
+        for idx, expected in [(idx1, exp1), (idx1, exp1), (idx1, exp1)]:
+            ordered = idx.sort_values()
+            self.assert_index_equal(ordered, expected)
+            self.assertIsNone(ordered.freq)
+
+            ordered = idx.sort_values(ascending=False)
+            self.assert_index_equal(ordered, expected[::-1])
+            self.assertIsNone(ordered.freq)
+
+            ordered, indexer = idx.sort_values(return_indexer=True)
+            self.assert_index_equal(ordered, expected)
+            self.assert_numpy_array_equal(indexer, np.array([0, 4, 3, 1, 2]))
+            self.assertIsNone(ordered.freq)
+
+            ordered, indexer = idx.sort_values(return_indexer=True, ascending=False)
+            self.assert_index_equal(ordered, expected[::-1])
+            self.assert_numpy_array_equal(indexer, np.array([2, 1, 3, 4, 0]))
+            self.assertIsNone(ordered.freq)
+
     def test_getitem(self):
         idx1 = pd.date_range('2011-01-01', '2011-01-31', freq='D', name='idx')
         idx2 = pd.date_range('2011-01-01', '2011-01-31', freq='D', tz='Asia/Tokyo', name='idx')
@@ -318,7 +430,7 @@ Freq: H"""
             self.assertEqual(result.freq, expected.freq)
 
             result = idx[-20:-5:3]
-            expected = pd.date_range('2011-01-12', '2011-01-25', freq='3D',
+            expected = pd.date_range('2011-01-12', '2011-01-24', freq='3D',
                                      tz=idx.tz, name='idx')
             self.assert_index_equal(result, expected)
             self.assertEqual(result.freq, expected.freq)
@@ -330,6 +442,68 @@ Freq: H"""
             self.assert_index_equal(result, expected)
             self.assertEqual(result.freq, expected.freq)
 
+    def test_drop_duplicates_metadata(self):
+        #GH 10115
+        idx = pd.date_range('2011-01-01', '2011-01-31', freq='D', name='idx')
+        result = idx.drop_duplicates()
+        self.assert_index_equal(idx, result)
+        self.assertEqual(idx.freq, result.freq)
+
+        idx_dup = idx.append(idx)
+        self.assertIsNone(idx_dup.freq)   # freq is reset
+        result = idx_dup.drop_duplicates()
+        self.assert_index_equal(idx, result)
+        self.assertIsNone(result.freq)
+
+    def test_take(self):
+        #GH 10295
+        idx1 = pd.date_range('2011-01-01', '2011-01-31', freq='D', name='idx')
+        idx2 = pd.date_range('2011-01-01', '2011-01-31', freq='D', tz='Asia/Tokyo', name='idx')
+
+        for idx in [idx1, idx2]:
+            result = idx.take([0])
+            self.assertEqual(result, pd.Timestamp('2011-01-01', tz=idx.tz))
+
+            result = idx.take([0, 1, 2])
+            expected = pd.date_range('2011-01-01', '2011-01-03', freq='D',
+                                     tz=idx.tz, name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+
+            result = idx.take([0, 2, 4])
+            expected = pd.date_range('2011-01-01', '2011-01-05', freq='2D',
+                                     tz=idx.tz, name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+
+            result = idx.take([7, 4, 1])
+            expected = pd.date_range('2011-01-08', '2011-01-02', freq='-3D',
+                                     tz=idx.tz, name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+
+            result = idx.take([3, 2, 5])
+            expected = DatetimeIndex(['2011-01-04', '2011-01-03', '2011-01-06'],
+                                     freq=None, tz=idx.tz, name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertIsNone(result.freq)
+
+            result = idx.take([-3, 2, 5])
+            expected = DatetimeIndex(['2011-01-29', '2011-01-03', '2011-01-06'],
+                                     freq=None, tz=idx.tz, name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertIsNone(result.freq)
+
+    def test_infer_freq(self):
+        # GH 11018
+        for freq in ['A', '2A', '-2A', 'Q', '-1Q', 'M', '-1M', 'D', '3D', '-3D',
+                     'W', '-1W', 'H', '2H', '-2H', 'T', '2T', 'S', '-3S']:
+            idx = pd.date_range('2011-01-01 09:00:00', freq=freq, periods=10)
+            result = pd.DatetimeIndex(idx.asi8, freq='infer')
+            tm.assert_index_equal(idx, result)
+            self.assertEqual(result.freq, freq)
+
+
 class TestTimedeltaIndexOps(Ops):
 
     def setUp(self):
@@ -416,6 +590,38 @@ class TestTimedeltaIndexOps(Ops):
                     result = getattr(idx, func)()
                     self.assertEqual(result, expected)
 
+    def test_representation_to_series(self):
+        idx1 = TimedeltaIndex([], freq='D')
+        idx2 = TimedeltaIndex(['1 days'], freq='D')
+        idx3 = TimedeltaIndex(['1 days', '2 days'], freq='D')
+        idx4 = TimedeltaIndex(['1 days', '2 days', '3 days'], freq='D')
+        idx5 = TimedeltaIndex(['1 days 00:00:01', '2 days', '3 days'])
+
+        exp1 = """Series([], dtype: timedelta64[ns])"""
+
+        exp2 = """0   1 days
+dtype: timedelta64[ns]"""
+
+        exp3 = """0   1 days
+1   2 days
+dtype: timedelta64[ns]"""
+
+        exp4 = """0   1 days
+1   2 days
+2   3 days
+dtype: timedelta64[ns]"""
+
+        exp5 = """0   1 days 00:00:01
+1   2 days 00:00:00
+2   3 days 00:00:00
+dtype: timedelta64[ns]"""
+
+        with pd.option_context('display.width',300):
+            for idx, expected in zip([idx1, idx2, idx3, idx4, idx5],
+                                     [exp1, exp2, exp3, exp4, exp5]):
+                result = repr(pd.Series(idx))
+                self.assertEqual(result, expected)
+
     def test_summary(self):
         # GH9116
         idx1 = TimedeltaIndex([], freq='D')
@@ -748,7 +954,7 @@ Freq: D"""
         tm.assert_index_equal(idx.unique(), expected)
 
         idx = TimedeltaIndex(['1 days 09:00:00', '1 days 09:00:00', '1 days 09:00:00',
-                             '1 days 08:00:00', '1 days 08:00:00', pd.NaT])
+                              '1 days 08:00:00', '1 days 08:00:00', pd.NaT])
 
         exp_idx = TimedeltaIndex(['1 days 09:00:00', '1 days 08:00:00'])
         expected = Series([3, 2], index=exp_idx)
@@ -774,6 +980,66 @@ Freq: D"""
         self.assertNotIn('foo',ts.__dict__.keys())
         self.assertRaises(AttributeError,lambda : ts.foo)
 
+    def test_order(self):
+        #GH 10295
+        idx1 = TimedeltaIndex(['1 day', '2 day', '3 day'], freq='D', name='idx')
+        idx2 = TimedeltaIndex(['1 hour', '2 hour', '3 hour'], freq='H', name='idx')
+
+        for idx in [idx1, idx2]:
+            ordered = idx.sort_values()
+            self.assert_index_equal(ordered, idx)
+            self.assertEqual(ordered.freq, idx.freq)
+
+            ordered = idx.sort_values(ascending=False)
+            expected = idx[::-1]
+            self.assert_index_equal(ordered, expected)
+            self.assertEqual(ordered.freq, expected.freq)
+            self.assertEqual(ordered.freq.n, -1)
+
+            ordered, indexer = idx.sort_values(return_indexer=True)
+            self.assert_index_equal(ordered, idx)
+            self.assert_numpy_array_equal(indexer, np.array([0, 1, 2]))
+            self.assertEqual(ordered.freq, idx.freq)
+
+            ordered, indexer = idx.sort_values(return_indexer=True, ascending=False)
+            self.assert_index_equal(ordered, idx[::-1])
+            self.assertEqual(ordered.freq, expected.freq)
+            self.assertEqual(ordered.freq.n, -1)
+
+        idx1 = TimedeltaIndex(['1 hour', '3 hour', '5 hour',
+                               '2 hour ', '1 hour'], name='idx1')
+        exp1 = TimedeltaIndex(['1 hour', '1 hour', '2 hour',
+                               '3 hour', '5 hour'], name='idx1')
+
+        idx2 = TimedeltaIndex(['1 day', '3 day', '5 day',
+                               '2 day', '1 day'], name='idx2')
+        exp2 = TimedeltaIndex(['1 day', '1 day', '2 day',
+                               '3 day', '5 day'], name='idx2')
+
+        idx3 = TimedeltaIndex([pd.NaT, '3 minute', '5 minute',
+                               '2 minute', pd.NaT], name='idx3')
+        exp3 = TimedeltaIndex([pd.NaT, pd.NaT, '2 minute', '3 minute',
+                               '5 minute'], name='idx3')
+
+        for idx, expected in [(idx1, exp1), (idx1, exp1), (idx1, exp1)]:
+            ordered = idx.sort_values()
+            self.assert_index_equal(ordered, expected)
+            self.assertIsNone(ordered.freq)
+
+            ordered = idx.sort_values(ascending=False)
+            self.assert_index_equal(ordered, expected[::-1])
+            self.assertIsNone(ordered.freq)
+
+            ordered, indexer = idx.sort_values(return_indexer=True)
+            self.assert_index_equal(ordered, expected)
+            self.assert_numpy_array_equal(indexer, np.array([0, 4, 3, 1, 2]))
+            self.assertIsNone(ordered.freq)
+
+            ordered, indexer = idx.sort_values(return_indexer=True, ascending=False)
+            self.assert_index_equal(ordered, expected[::-1])
+            self.assert_numpy_array_equal(indexer, np.array([2, 1, 3, 4, 0]))
+            self.assertIsNone(ordered.freq)
+
     def test_getitem(self):
         idx1 = pd.timedelta_range('1 day', '31 day', freq='D', name='idx')
 
@@ -792,7 +1058,7 @@ Freq: D"""
             self.assertEqual(result.freq, expected.freq)
 
             result = idx[-20:-5:3]
-            expected = pd.timedelta_range('12 day', '25 day', freq='3D', name='idx')
+            expected = pd.timedelta_range('12 day', '24 day', freq='3D', name='idx')
             self.assert_index_equal(result, expected)
             self.assertEqual(result.freq, expected.freq)
 
@@ -802,6 +1068,64 @@ Freq: D"""
             self.assert_index_equal(result, expected)
             self.assertEqual(result.freq, expected.freq)
 
+    def test_drop_duplicates_metadata(self):
+        #GH 10115
+        idx = pd.timedelta_range('1 day', '31 day', freq='D', name='idx')
+        result = idx.drop_duplicates()
+        self.assert_index_equal(idx, result)
+        self.assertEqual(idx.freq, result.freq)
+
+        idx_dup = idx.append(idx)
+        self.assertIsNone(idx_dup.freq)   # freq is reset
+        result = idx_dup.drop_duplicates()
+        self.assert_index_equal(idx, result)
+        self.assertIsNone(result.freq)
+
+    def test_take(self):
+        #GH 10295
+        idx1 = pd.timedelta_range('1 day', '31 day', freq='D', name='idx')
+
+        for idx in [idx1]:
+            result = idx.take([0])
+            self.assertEqual(result, pd.Timedelta('1 day'))
+
+            result = idx.take([-1])
+            self.assertEqual(result, pd.Timedelta('31 day'))
+
+            result = idx.take([0, 1, 2])
+            expected = pd.timedelta_range('1 day', '3 day', freq='D', name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+
+            result = idx.take([0, 2, 4])
+            expected = pd.timedelta_range('1 day', '5 day', freq='2D', name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+
+            result = idx.take([7, 4, 1])
+            expected = pd.timedelta_range('8 day', '2 day', freq='-3D', name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+
+            result = idx.take([3, 2, 5])
+            expected = TimedeltaIndex(['4 day', '3 day', '6 day'], name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertIsNone(result.freq)
+
+            result = idx.take([-3, 2, 5])
+            expected = TimedeltaIndex(['29 day', '3 day', '6 day'], name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertIsNone(result.freq)
+
+    def test_infer_freq(self):
+        # GH 11018
+        for freq in ['D', '3D', '-3D', 'H', '2H', '-2H', 'T', '2T', 'S', '-3S']:
+            idx = pd.timedelta_range('1', freq=freq, periods=10)
+            result = pd.TimedeltaIndex(idx.asi8, freq='infer')
+            tm.assert_index_equal(idx, result)
+            self.assertEqual(result.freq, freq)
+
+
 class TestPeriodIndexOps(Ops):
 
     def setUp(self):
@@ -915,6 +1239,60 @@ class TestPeriodIndexOps(Ops):
                 result = getattr(idx, func)()
                 self.assertEqual(result, expected)
 
+    def test_representation_to_series(self):
+        # GH 10971
+        idx1 = PeriodIndex([], freq='D')
+        idx2 = PeriodIndex(['2011-01-01'], freq='D')
+        idx3 = PeriodIndex(['2011-01-01', '2011-01-02'], freq='D')
+        idx4 = PeriodIndex(['2011-01-01', '2011-01-02', '2011-01-03'], freq='D')
+        idx5 = PeriodIndex(['2011', '2012', '2013'], freq='A')
+        idx6 = PeriodIndex(['2011-01-01 09:00', '2012-02-01 10:00', 'NaT'], freq='H')
+
+        idx7 = pd.period_range('2013Q1', periods=1, freq="Q")
+        idx8 = pd.period_range('2013Q1', periods=2, freq="Q")
+        idx9 = pd.period_range('2013Q1', periods=3, freq="Q")
+
+        exp1 = """Series([], dtype: object)"""
+
+        exp2 = """0   2011-01-01
+dtype: object"""
+
+        exp3 = """0   2011-01-01
+1   2011-01-02
+dtype: object"""
+
+        exp4 = """0   2011-01-01
+1   2011-01-02
+2   2011-01-03
+dtype: object"""
+
+        exp5 = """0   2011
+1   2012
+2   2013
+dtype: object"""
+
+        exp6 = """0   2011-01-01 09:00
+1   2012-02-01 10:00
+2                NaT
+dtype: object"""
+
+        exp7 = """0   2013Q1
+dtype: object"""
+
+        exp8 = """0   2013Q1
+1   2013Q2
+dtype: object"""
+
+        exp9 = """0   2013Q1
+1   2013Q2
+2   2013Q3
+dtype: object"""
+
+        for idx, expected in zip([idx1, idx2, idx3, idx4, idx5, idx6, idx7, idx8, idx9],
+                                 [exp1, exp2, exp3, exp4, exp5, exp6, exp7, exp8, exp9]):
+            result = repr(pd.Series(idx))
+            self.assertEqual(result, expected)
+
     def test_summary(self):
         # GH9116
         idx1 = PeriodIndex([], freq='D')
@@ -1030,6 +1408,7 @@ Freq: Q-DEC"""
 
         for o in [pd.offsets.YearBegin(2), pd.offsets.MonthBegin(1), pd.offsets.Minute(),
                   np.timedelta64(365, 'D'), timedelta(365), Timedelta(days=365)]:
+            msg = 'Input has different freq from PeriodIndex\\(freq=A-DEC\\)'
             with tm.assertRaisesRegexp(ValueError, 'Input has different freq from Period'):
                 rng + o
 
@@ -1043,7 +1422,8 @@ Freq: Q-DEC"""
         for o in [pd.offsets.YearBegin(2), pd.offsets.MonthBegin(1), pd.offsets.Minute(),
                   np.timedelta64(365, 'D'), timedelta(365),  Timedelta(days=365)]:
             rng = pd.period_range('2014-01', '2016-12', freq='M')
-            with tm.assertRaisesRegexp(ValueError, 'Input has different freq from Period'):
+            msg = 'Input has different freq from PeriodIndex\\(freq=M\\)'
+            with tm.assertRaisesRegexp(ValueError, msg):
                 rng + o
 
         # Tick
@@ -1061,7 +1441,8 @@ Freq: Q-DEC"""
         for o in [pd.offsets.YearBegin(2), pd.offsets.MonthBegin(1), pd.offsets.Minute(),
                   np.timedelta64(4, 'h'), timedelta(hours=23), Timedelta('23:00:00')]:
             rng = pd.period_range('2014-05-01', '2014-05-15', freq='D')
-            with tm.assertRaisesRegexp(ValueError, 'Input has different freq from Period'):
+            msg = 'Input has different freq from PeriodIndex\\(freq=D\\)'
+            with tm.assertRaisesRegexp(ValueError, msg):
                 rng + o
 
         offsets = [pd.offsets.Hour(2), timedelta(hours=2), np.timedelta64(2, 'h'),
@@ -1078,9 +1459,10 @@ Freq: Q-DEC"""
         for delta in [pd.offsets.YearBegin(2), timedelta(minutes=30),
                       np.timedelta64(30, 's'),  Timedelta(seconds=30)]:
             rng = pd.period_range('2014-01-01 10:00', '2014-01-05 10:00', freq='H')
-            with tm.assertRaisesRegexp(ValueError, 'Input has different freq from Period'):
+            msg = 'Input has different freq from PeriodIndex\\(freq=H\\)'
+            with tm.assertRaisesRegexp(ValueError, msg):
                 result = rng + delta
-            with tm.assertRaisesRegexp(ValueError, 'Input has different freq from Period'):
+            with tm.assertRaisesRegexp(ValueError, msg):
                 rng += delta
 
         # int
@@ -1141,7 +1523,8 @@ Freq: Q-DEC"""
         for o in [pd.offsets.YearBegin(2), pd.offsets.MonthBegin(1), pd.offsets.Minute(),
                   np.timedelta64(365, 'D'), timedelta(365)]:
             rng = pd.period_range('2014', '2024', freq='A')
-            with tm.assertRaisesRegexp(ValueError, 'Input has different freq from Period'):
+            msg = 'Input has different freq from PeriodIndex\\(freq=A-DEC\\)'
+            with tm.assertRaisesRegexp(ValueError, msg):
                 rng - o
 
         rng = pd.period_range('2014-01', '2016-12', freq='M')
@@ -1154,7 +1537,8 @@ Freq: Q-DEC"""
         for o in [pd.offsets.YearBegin(2), pd.offsets.MonthBegin(1), pd.offsets.Minute(),
                   np.timedelta64(365, 'D'), timedelta(365)]:
             rng = pd.period_range('2014-01', '2016-12', freq='M')
-            with tm.assertRaisesRegexp(ValueError, 'Input has different freq from Period'):
+            msg = 'Input has different freq from PeriodIndex\\(freq=M\\)'
+            with tm.assertRaisesRegexp(ValueError, msg):
                 rng - o
 
         # Tick
@@ -1171,7 +1555,8 @@ Freq: Q-DEC"""
         for o in [pd.offsets.YearBegin(2), pd.offsets.MonthBegin(1), pd.offsets.Minute(),
                   np.timedelta64(4, 'h'), timedelta(hours=23)]:
             rng = pd.period_range('2014-05-01', '2014-05-15', freq='D')
-            with tm.assertRaisesRegexp(ValueError, 'Input has different freq from Period'):
+            msg = 'Input has different freq from PeriodIndex\\(freq=D\\)'
+            with tm.assertRaisesRegexp(ValueError, msg):
                 rng - o
 
         offsets = [pd.offsets.Hour(2), timedelta(hours=2), np.timedelta64(2, 'h'),
@@ -1186,9 +1571,10 @@ Freq: Q-DEC"""
 
         for delta in [pd.offsets.YearBegin(2), timedelta(minutes=30), np.timedelta64(30, 's')]:
             rng = pd.period_range('2014-01-01 10:00', '2014-01-05 10:00', freq='H')
-            with tm.assertRaisesRegexp(ValueError, 'Input has different freq from Period'):
+            msg = 'Input has different freq from PeriodIndex\\(freq=H\\)'
+            with tm.assertRaisesRegexp(ValueError, msg):
                 result = rng + delta
-            with tm.assertRaisesRegexp(ValueError, 'Input has different freq from Period'):
+            with tm.assertRaisesRegexp(ValueError, msg):
                 rng += delta
 
         # int
@@ -1228,6 +1614,233 @@ Freq: Q-DEC"""
 
         tm.assert_index_equal(idx.unique(), exp_idx)
 
+    def test_drop_duplicates_metadata(self):
+        #GH 10115
+        idx = pd.period_range('2011-01-01', '2011-01-31', freq='D', name='idx')
+        result = idx.drop_duplicates()
+        self.assert_index_equal(idx, result)
+        self.assertEqual(idx.freq, result.freq)
+
+        idx_dup = idx.append(idx) # freq will not be reset
+        result = idx_dup.drop_duplicates()
+        self.assert_index_equal(idx, result)
+        self.assertEqual(idx.freq, result.freq)
+
+    def test_order_compat(self):
+
+        def _check_freq(index, expected_index):
+            if isinstance(index, PeriodIndex):
+                self.assertEqual(index.freq, expected_index.freq)
+
+        pidx = PeriodIndex(['2011', '2012', '2013'], name='pidx', freq='A')
+        # for compatibility check
+        iidx = Index([2011, 2012, 2013], name='idx')
+        for idx in [pidx, iidx]:
+            ordered = idx.sort_values()
+            self.assert_index_equal(ordered, idx)
+            _check_freq(ordered, idx)
+
+            ordered = idx.sort_values(ascending=False)
+            self.assert_index_equal(ordered, idx[::-1])
+            _check_freq(ordered, idx[::-1])
+
+            ordered, indexer = idx.sort_values(return_indexer=True)
+            self.assert_index_equal(ordered, idx)
+            self.assert_numpy_array_equal(indexer, np.array([0, 1, 2]))
+            _check_freq(ordered, idx)
+
+            ordered, indexer = idx.sort_values(return_indexer=True, ascending=False)
+            self.assert_index_equal(ordered, idx[::-1])
+            self.assert_numpy_array_equal(indexer, np.array([2, 1, 0]))
+            _check_freq(ordered, idx[::-1])
+
+        pidx = PeriodIndex(['2011', '2013', '2015', '2012', '2011'], name='pidx', freq='A')
+        pexpected = PeriodIndex(['2011', '2011', '2012', '2013', '2015'], name='pidx', freq='A')
+        # for compatibility check
+        iidx = Index([2011, 2013, 2015, 2012, 2011], name='idx')
+        iexpected = Index([2011, 2011, 2012, 2013, 2015], name='idx')
+        for idx, expected in [(pidx, pexpected), (iidx, iexpected)]:
+            ordered = idx.sort_values()
+            self.assert_index_equal(ordered, expected)
+            _check_freq(ordered, idx)
+
+            ordered = idx.sort_values(ascending=False)
+            self.assert_index_equal(ordered, expected[::-1])
+            _check_freq(ordered, idx)
+
+            ordered, indexer = idx.sort_values(return_indexer=True)
+            self.assert_index_equal(ordered, expected)
+            self.assert_numpy_array_equal(indexer, np.array([0, 4, 3, 1, 2]))
+            _check_freq(ordered, idx)
+
+            ordered, indexer = idx.sort_values(return_indexer=True, ascending=False)
+            self.assert_index_equal(ordered, expected[::-1])
+            self.assert_numpy_array_equal(indexer, np.array([2, 1, 3, 4, 0]))
+            _check_freq(ordered, idx)
+
+        pidx = PeriodIndex(['2011', '2013', 'NaT', '2011'], name='pidx', freq='D')
+
+        result = pidx.sort_values()
+        expected = PeriodIndex(['NaT', '2011', '2011', '2013'], name='pidx', freq='D')
+        self.assert_index_equal(result, expected)
+        self.assertEqual(result.freq, 'D')
+
+        result = pidx.sort_values(ascending=False)
+        expected = PeriodIndex(['2013', '2011', '2011', 'NaT'], name='pidx', freq='D')
+        self.assert_index_equal(result, expected)
+        self.assertEqual(result.freq, 'D')
+
+    def test_order(self):
+        for freq in ['D', '2D', '4D']:
+            idx = PeriodIndex(['2011-01-01', '2011-01-02', '2011-01-03'],
+                              freq=freq, name='idx')
+
+            ordered = idx.sort_values()
+            self.assert_index_equal(ordered, idx)
+            self.assertEqual(ordered.freq, idx.freq)
+
+            ordered = idx.sort_values(ascending=False)
+            expected = idx[::-1]
+            self.assert_index_equal(ordered, expected)
+            self.assertEqual(ordered.freq, expected.freq)
+            self.assertEqual(ordered.freq, freq)
+
+            ordered, indexer = idx.sort_values(return_indexer=True)
+            self.assert_index_equal(ordered, idx)
+            self.assert_numpy_array_equal(indexer, np.array([0, 1, 2]))
+            self.assertEqual(ordered.freq, idx.freq)
+            self.assertEqual(ordered.freq, freq)
+
+            ordered, indexer = idx.sort_values(return_indexer=True, ascending=False)
+            expected = idx[::-1]
+            self.assert_index_equal(ordered, expected)
+            self.assert_numpy_array_equal(indexer, np.array([2, 1, 0]))
+            self.assertEqual(ordered.freq, expected.freq)
+            self.assertEqual(ordered.freq, freq)
+
+        idx1 = PeriodIndex(['2011-01-01', '2011-01-03', '2011-01-05',
+                            '2011-01-02', '2011-01-01'], freq='D', name='idx1')
+        exp1 = PeriodIndex(['2011-01-01', '2011-01-01', '2011-01-02',
+                            '2011-01-03', '2011-01-05'], freq='D', name='idx1')
+
+        idx2 = PeriodIndex(['2011-01-01', '2011-01-03', '2011-01-05',
+                            '2011-01-02', '2011-01-01'],
+                           freq='D',  name='idx2')
+        exp2 = PeriodIndex(['2011-01-01', '2011-01-01', '2011-01-02',
+                            '2011-01-03', '2011-01-05'],
+                           freq='D', name='idx2')
+
+        idx3 = PeriodIndex([pd.NaT, '2011-01-03', '2011-01-05',
+                            '2011-01-02', pd.NaT], freq='D', name='idx3')
+        exp3 = PeriodIndex([pd.NaT, pd.NaT, '2011-01-02', '2011-01-03',
+                            '2011-01-05'], freq='D', name='idx3')
+
+        for idx, expected in [(idx1, exp1), (idx1, exp1), (idx1, exp1)]:
+            ordered = idx.sort_values()
+            self.assert_index_equal(ordered, expected)
+            self.assertEqual(ordered.freq, 'D')
+
+            ordered = idx.sort_values(ascending=False)
+            self.assert_index_equal(ordered, expected[::-1])
+            self.assertEqual(ordered.freq, 'D')
+
+            ordered, indexer = idx.sort_values(return_indexer=True)
+            self.assert_index_equal(ordered, expected)
+            self.assert_numpy_array_equal(indexer, np.array([0, 4, 3, 1, 2]))
+            self.assertEqual(ordered.freq, 'D')
+
+            ordered, indexer = idx.sort_values(return_indexer=True, ascending=False)
+            self.assert_index_equal(ordered, expected[::-1])
+            self.assert_numpy_array_equal(indexer, np.array([2, 1, 3, 4, 0]))
+            self.assertEqual(ordered.freq, 'D')
+
+    def test_getitem(self):
+        idx1 = pd.period_range('2011-01-01', '2011-01-31', freq='D', name='idx')
+
+        for idx in [idx1]:
+            result = idx[0]
+            self.assertEqual(result, pd.Period('2011-01-01', freq='D'))
+
+            result = idx[-1]
+            self.assertEqual(result, pd.Period('2011-01-31', freq='D'))
+
+            result = idx[0:5]
+            expected = pd.period_range('2011-01-01', '2011-01-05', freq='D',
+                                       name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+            self.assertEqual(result.freq, 'D')
+
+            result = idx[0:10:2]
+            expected = pd.PeriodIndex(['2011-01-01', '2011-01-03', '2011-01-05',
+                                       '2011-01-07', '2011-01-09'],
+                                      freq='D', name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+            self.assertEqual(result.freq, 'D')
+
+            result = idx[-20:-5:3]
+            expected = pd.PeriodIndex(['2011-01-12', '2011-01-15', '2011-01-18',
+                                       '2011-01-21', '2011-01-24'],
+                                      freq='D', name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+            self.assertEqual(result.freq, 'D')
+
+            result = idx[4::-1]
+            expected = PeriodIndex(['2011-01-05', '2011-01-04', '2011-01-03',
+                                   '2011-01-02', '2011-01-01'],
+                                   freq='D', name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+            self.assertEqual(result.freq, 'D')
+
+    def test_take(self):
+        #GH 10295
+        idx1 = pd.period_range('2011-01-01', '2011-01-31', freq='D', name='idx')
+
+        for idx in [idx1]:
+            result = idx.take([0])
+            self.assertEqual(result, pd.Period('2011-01-01', freq='D'))
+
+            result = idx.take([5])
+            self.assertEqual(result, pd.Period('2011-01-06', freq='D'))
+
+            result = idx.take([0, 1, 2])
+            expected = pd.period_range('2011-01-01', '2011-01-03', freq='D',
+                                       name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, 'D')
+            self.assertEqual(result.freq, expected.freq)
+
+            result = idx.take([0, 2, 4])
+            expected = pd.PeriodIndex(['2011-01-01', '2011-01-03', '2011-01-05'],
+                                      freq='D', name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+            self.assertEqual(result.freq, 'D')
+
+            result = idx.take([7, 4, 1])
+            expected = pd.PeriodIndex(['2011-01-08', '2011-01-05', '2011-01-02'],
+                                      freq='D', name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+            self.assertEqual(result.freq, 'D')
+
+            result = idx.take([3, 2, 5])
+            expected = PeriodIndex(['2011-01-04', '2011-01-03', '2011-01-06'],
+                                   freq='D', name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+            self.assertEqual(result.freq, 'D')
+
+            result = idx.take([-3, 2, 5])
+            expected = PeriodIndex(['2011-01-29', '2011-01-03', '2011-01-06'],
+                                   freq='D', name='idx')
+            self.assert_index_equal(result, expected)
+            self.assertEqual(result.freq, expected.freq)
+            self.assertEqual(result.freq, 'D')
+
 
 if __name__ == '__main__':
     import nose
