@@ -21,14 +21,17 @@ from distutils.version import LooseVersion
 
 from numpy.random import randn, rand
 import numpy as np
-from numpy.testing import assert_array_equal
 
 import pandas as pd
-from pandas.core.common import is_sequence, array_equivalent, is_list_like, is_number
+from pandas.core.common import (is_sequence, array_equivalent, is_list_like, is_number,
+                                is_datetimelike_v_numeric, is_datetimelike_v_object,
+                                is_number, pprint_thing, take_1d,
+                                needs_i8_conversion)
+
 import pandas.compat as compat
 from pandas.compat import(
     filter, map, zip, range, unichr, lrange, lmap, lzip, u, callable, Counter,
-    raise_with_traceback, httplib
+    raise_with_traceback, httplib, is_platform_windows, is_platform_32bit
 )
 
 from pandas.computation import expressions as expr
@@ -59,7 +62,6 @@ def reset_testing_mode():
 
 set_testing_mode()
 
-
 class TestCase(unittest.TestCase):
 
     @classmethod
@@ -174,8 +176,7 @@ def close(fignum=None):
 
 def _skip_if_32bit():
     import nose
-    import struct
-    if struct.calcsize("P") * 8 < 64:
+    if is_platform_32bit():
         raise nose.SkipTest("skipping for 32 bit")
 
 def mplskip(cls):
@@ -194,6 +195,14 @@ def mplskip(cls):
     return cls
 
 
+def _skip_if_mpl_1_5():
+    import matplotlib
+    v = matplotlib.__version__
+    if v > LooseVersion('1.4.3') or v[0] == '0':
+        import nose
+        raise nose.SkipTest("matplotlib 1.5")
+
+
 def _skip_if_no_scipy():
     try:
         import scipy.stats
@@ -223,6 +232,17 @@ def _skip_if_no_dateutil():
         raise nose.SkipTest("dateutil not installed")
 
 
+def _skip_if_windows_python_3():
+    if compat.PY3 and is_platform_windows():
+        import nose
+        raise nose.SkipTest("not used on python 3/win32")
+
+def _skip_if_windows():
+    if is_platform_windows():
+        import nose
+        raise nose.SkipTest("Running on Windows")
+
+
 def _skip_if_no_cday():
     from pandas.core.datetools import cday
     if cday is None:
@@ -230,6 +250,44 @@ def _skip_if_no_cday():
         raise nose.SkipTest("CustomBusinessDay not available.")
 
 
+def _skip_if_python26():
+    if sys.version_info[:2] == (2, 6):
+        import nose
+        raise nose.SkipTest("skipping on python2.6")
+
+def _incompat_bottleneck_version(method):
+    """ skip if we have bottleneck installed
+    and its >= 1.0
+    as we don't match the nansum/nanprod behavior for all-nan
+    ops, see GH9422
+    """
+    if method not in ['sum','prod']:
+        return False
+    try:
+        import bottleneck as bn
+        return bn.__version__ >= LooseVersion('1.0')
+    except ImportError:
+        return False
+
+def skip_if_no_ne(engine='numexpr'):
+    import nose
+    _USE_NUMEXPR = pd.computation.expressions._USE_NUMEXPR
+
+    if engine == 'numexpr':
+        try:
+            import numexpr as ne
+        except ImportError:
+            raise nose.SkipTest("numexpr not installed")
+
+        if not _USE_NUMEXPR:
+            raise nose.SkipTest("numexpr disabled")
+
+        if ne.__version__ < LooseVersion('2.0'):
+            raise nose.SkipTest("numexpr version too low: "
+                                "%s" % ne.__version__)
+
+
+
 #------------------------------------------------------------------------------
 # locale utilities
 
@@ -511,13 +569,6 @@ def equalContents(arr1, arr2):
     return frozenset(arr1) == frozenset(arr2)
 
 
-def assert_isinstance(obj, class_type_or_tuple, msg=''):
-    """asserts that obj is an instance of class_type_or_tuple"""
-    assert isinstance(obj, class_type_or_tuple), (
-        "%sExpected object to be of type %r, found %r instead" % (
-            msg, class_type_or_tuple, type(obj)))
-
-
 def assert_equal(a, b, msg=""):
     """asserts that a equals b, like nose's assert_equal, but allows custom message to start.
     Passes a and b to format string as well. So you can use '{0}' and '{1}' to display a and b.
@@ -533,23 +584,128 @@ def assert_equal(a, b, msg=""):
     assert a == b, "%s: %r != %r" % (msg.format(a,b), a, b)
 
 
-def assert_index_equal(left, right, exact=False, check_names=True):
-    assert_isinstance(left, Index, '[index] ')
-    assert_isinstance(right, Index, '[index] ')
-    if not left.equals(right) or (exact and type(left) != type(right)):
-        raise AssertionError("[index] left [{0} {1}], right [{2} {3}]".format(left.dtype,
-                                                                              left,
-                                                                              right,
-                                                                              right.dtype))
+def assert_index_equal(left, right, exact=False, check_names=True,
+                       check_less_precise=False, check_exact=True, obj='Index'):
+    """Check that left and right Index are equal.
+
+    Parameters
+    ----------
+    left : Index
+    right : Index
+    exact : bool, default False
+        Whether to check the Index class, dtype and inferred_type are identical.
+    check_names : bool, default True
+        Whether to check the names attribute.
+    check_less_precise : bool, default False
+        Specify comparison precision. Only used when check_exact is False.
+        5 digits (False) or 3 digits (True) after decimal points are compared.
+    check_exact : bool, default True
+        Whether to compare number exactly.
+    obj : str, default 'Index'
+        Specify object name being compared, internally used to show appropriate
+        assertion message
+    """
+
+    def _check_types(l, r, obj='Index'):
+        if exact:
+            if type(l) != type(r):
+                msg = '{0} classes are different'.format(obj)
+                raise_assert_detail(obj, msg, l, r)
+            assert_attr_equal('dtype', l, r, obj=obj)
+            assert_attr_equal('inferred_type', l, r, obj=obj)
+
+    def _get_ilevel_values(index, level):
+        # accept level number only
+        unique = index.levels[level]
+        labels = index.labels[level]
+        filled = take_1d(unique.values, labels, fill_value=unique._na_value)
+        values = unique._simple_new(filled, index.names[level],
+                                    freq=getattr(unique, 'freq', None),
+                                    tz=getattr(unique, 'tz', None))
+        return values
+
+    # instance validation
+    assertIsInstance(left, Index, '[index] ')
+    assertIsInstance(right, Index, '[index] ')
+
+    # class / dtype comparison
+    _check_types(left, right)
+
+    # level comparison
+    if left.nlevels != right.nlevels:
+        raise_assert_detail(obj, '{0} levels are different'.format(obj),
+                            '{0}, {1}'.format(left.nlevels, left),
+                            '{0}, {1}'.format(right.nlevels, right))
+
+    # length comparison
+    if len(left) != len(right):
+        raise_assert_detail(obj, '{0} length are different'.format(obj),
+                           '{0}, {1}'.format(len(left), left),
+                           '{0}, {1}'.format(len(right), right))
+
+    # MultiIndex special comparison for little-friendly error messages
+    if left.nlevels > 1:
+        for level in range(left.nlevels):
+            # cannot use get_level_values here because it can change dtype
+            llevel = _get_ilevel_values(left, level)
+            rlevel = _get_ilevel_values(right, level)
+
+            lobj = 'MultiIndex level [{0}]'.format(level)
+            assert_index_equal(llevel, rlevel,
+                               exact=exact, check_names=check_names,
+                               check_less_precise=check_less_precise,
+                               check_exact=check_exact, obj=lobj)
+            # get_level_values may change dtype
+            _check_types(left.levels[level], right.levels[level], obj=obj)
+
+    if check_exact:
+        if not left.equals(right):
+            diff = np.sum((left.values != right.values).astype(int)) * 100.0 / len(left)
+            msg = '{0} values are different ({1} %)'.format(obj, np.round(diff, 5))
+            raise_assert_detail(obj, msg, left, right)
+    else:
+        assert_almost_equal(left.values, right.values,
+                            check_less_precise=check_less_precise,
+                            obj=obj, lobj=left, robj=right)
+
+    # metadata comparison
     if check_names:
-        assert_attr_equal('names', left, right)
+        assert_attr_equal('names', left, right, obj=obj)
 
 
-def assert_attr_equal(attr, left, right):
-    """checks attributes are equal. Both objects must have attribute."""
+def assert_attr_equal(attr, left, right, obj='Attributes'):
+    """checks attributes are equal. Both objects must have attribute.
+
+    Parameters
+    ----------
+    attr : str
+        Attribute name being compared.
+    left : object
+    right : object
+    obj : str, default 'Attributes'
+        Specify object name being compared, internally used to show appropriate
+        assertion message
+    """
+
     left_attr = getattr(left, attr)
     right_attr = getattr(right, attr)
-    assert_equal(left_attr,right_attr,"attr is not equal [{0}]" .format(attr))
+
+    if left_attr is right_attr:
+        return True
+    elif (is_number(left_attr) and np.isnan(left_attr) and
+          is_number(right_attr) and np.isnan(right_attr)):
+        # np.nan
+        return True
+
+    result = left_attr == right_attr
+    if not isinstance(result, bool):
+        result = result.all()
+
+    if result:
+        return True
+    else:
+        raise_assert_detail(obj, 'Attribute "{0}" are different'.format(attr),
+                        left_attr, right_attr)
 
 
 def isiterable(obj):
@@ -601,6 +757,9 @@ def assertIsInstance(obj, cls, msg=''):
         "%sExpected object to be of type %r, found %r instead" % (
             msg, cls, type(obj)))
 
+def assert_isinstance(obj, class_type_or_tuple, msg=''):
+    return deprecate('assert_isinstance', assertIsInstance)(obj, class_type_or_tuple, msg=msg)
+
 
 def assertNotIsInstance(obj, cls, msg=''):
     """Test that obj is not an instance of cls
@@ -624,42 +783,68 @@ def assert_categorical_equal(res, exp):
     if res.ordered != exp.ordered:
         raise AssertionError("ordered not the same")
 
-    if res.name != exp.name:
-        raise AssertionError("name not the same")
+
+def raise_assert_detail(obj, message, left, right):
+    if isinstance(left, np.ndarray):
+        left = pprint_thing(left)
+    if isinstance(right, np.ndarray):
+        right = pprint_thing(right)
+
+    msg = """{0} are different
+
+{1}
+[left]:  {2}
+[right]: {3}""".format(obj, message, left, right)
+    raise AssertionError(msg)
 
 
-def assert_numpy_array_equal(np_array, assert_equal):
-    """Checks that 'np_array' is equal to 'assert_equal'
+def assert_numpy_array_equal(left, right,
+                             strict_nan=False, err_msg=None,
+                             obj='numpy array'):
+    """Checks that 'np_array' is equivalent to 'assert_equal'.
 
-    Note that the expected array should not contain `np.nan`!
-    Two numpy arrays are equal if all
-    elements are equal, which is not possible if `np.nan` is such an element!
-
-    If the expected array includes `np.nan` use
-    `assert_numpy_array_equivalent(...)`.
+    This is similar to ``numpy.testing.assert_array_equal``, but can
+    check equality including ``np.nan``. Two numpy arrays are regarded as
+    equivalent if the arrays have equal non-NaN elements,
+    and `np.nan` in corresponding locations.
     """
-    if np.array_equal(np_array, assert_equal):
+
+    # compare shape and values
+    if array_equivalent(left, right, strict_nan=strict_nan):
         return
-    raise AssertionError(
-        '{0} is not equal to {1}.'.format(np_array, assert_equal))
 
+    if err_msg is None:
+        # show detailed error
 
-def assert_numpy_array_equivalent(np_array, assert_equal, strict_nan=False):
-    """Checks that 'np_array' is equivalent to 'assert_equal'
+        if np.isscalar(left) and np.isscalar(right):
+            # show scalar comparison error
+            assert_equal(left, right)
+        elif is_list_like(left) and is_list_like(right):
+            # some test cases pass list
+            left = np.asarray(left)
+            right = np.array(right)
 
-    Two numpy arrays are equivalent if the arrays have equal non-NaN elements,
-     and `np.nan` in corresponding locations.
+            if left.shape != right.shape:
+                raise_assert_detail(obj, '{0} shapes are different'.format(obj),
+                                    left.shape, right.shape)
 
-    If the the expected array does not contain `np.nan`
-    `assert_numpy_array_equivalent` is the similar to
-    `assert_numpy_array_equal()`. If the expected array includes
-    `np.nan` use this
-    function.
-    """
-    if array_equivalent(np_array, assert_equal, strict_nan=strict_nan):
-        return
-    raise AssertionError(
-        '{0} is not equivalent to {1}.'.format(np_array, assert_equal))
+            diff = 0
+            for l, r in zip(left, right):
+                # count up differences
+                if not array_equivalent(l, r, strict_nan=strict_nan):
+                    diff += 1
+
+            diff = diff * 100.0 / left.size
+            msg = '{0} values are different ({1} %)'.format(obj, np.round(diff, 5))
+            raise_assert_detail(obj, msg, left, right)
+        elif is_list_like(left):
+            msg = "First object is iterable, second isn't"
+            raise_assert_detail(obj, msg, left, right)
+        else:
+            msg = "Second object is iterable, first isn't"
+            raise_assert_detail(obj, msg, left, right)
+
+    raise AssertionError(err_msg)
 
 
 # This could be refactored to use the NDFrame.equals method
@@ -667,38 +852,81 @@ def assert_series_equal(left, right, check_dtype=True,
                         check_index_type=False,
                         check_series_type=False,
                         check_less_precise=False,
+                        check_names=True,
                         check_exact=False,
-                        check_names=True):
+                        check_datetimelike_compat=False,
+                        obj='Series'):
+
+    """Check that left and right Series are equal.
+
+    Parameters
+    ----------
+    left : Series
+    right : Series
+    check_dtype : bool, default True
+        Whether to check the Series dtype is identical.
+    check_index_type : bool, default False
+        Whether to check the Index class, dtype and inferred_type are identical.
+    check_series_type : bool, default False
+        Whether to check the Series class is identical.
+    check_less_precise : bool, default False
+        Specify comparison precision. Only used when check_exact is False.
+        5 digits (False) or 3 digits (True) after decimal points are compared.
+    check_exact : bool, default False
+        Whether to compare number exactly.
+    check_names : bool, default True
+        Whether to check the Series and Index names attribute.
+    check_dateteimelike_compat : bool, default False
+        Compare datetime-like which is comparable ignoring dtype.
+    obj : str, default 'Series'
+        Specify object name being compared, internally used to show appropriate
+        assertion message
+    """
+
+    # instance validation
+    assertIsInstance(left, Series, '[Series] ')
+    assertIsInstance(right, Series, '[Series] ')
+
     if check_series_type:
-        assert_isinstance(left, type(right))
+        assertIsInstance(left, type(right))
+
+    # length comparison
+    if len(left) != len(right):
+        raise_assert_detail(obj, 'Series length are different',
+                            '{0}, {1}'.format(len(left), left.index),
+                            '{0}, {1}'.format(len(right), right.index))
+
+    # index comparison
+    assert_index_equal(left.index, right.index, exact=check_index_type,
+                       check_names=check_names,
+                       check_less_precise=check_less_precise, check_exact=check_exact,
+                       obj='{0}.index'.format(obj))
+
     if check_dtype:
         assert_attr_equal('dtype', left, right)
+
     if check_exact:
-        if not np.array_equal(left.values, right.values):
-            raise AssertionError('{0} is not equal to {1}.'.format(left.values,
-                                                                   right.values))
-    else:
-        assert_almost_equal(left.values, right.values, check_less_precise)
-    if check_less_precise:
-        assert_almost_equal(
-            left.index.values, right.index.values, check_less_precise)
-    else:
-        assert_index_equal(left.index, right.index, check_names=check_names)
-    if check_index_type:
-        for level in range(left.index.nlevels):
-            lindex = left.index.get_level_values(level)
-            rindex = right.index.get_level_values(level)
-            assert_isinstance(lindex, type(rindex))
-            assert_attr_equal('dtype', lindex, rindex)
-            assert_attr_equal('inferred_type', lindex, rindex)
-    if check_names:
-        if is_number(left.name) and np.isnan(left.name):
-            # Series.name can be np.nan in some test cases
-            assert is_number(right.name) and np.isnan(right.name)
-        elif left.name is pd.NaT:
-            assert right.name is pd.NaT
+        assert_numpy_array_equal(left.get_values(), right.get_values(),
+                                 obj='{0}'.format(obj))
+    elif check_datetimelike_compat:
+        # we want to check only if we have compat dtypes
+        # e.g. integer and M|m are NOT compat, but we can simply check the values in that case
+        if is_datetimelike_v_numeric(left, right) or is_datetimelike_v_object(left, right) or needs_i8_conversion(left) or needs_i8_conversion(right):
+
+            # datetimelike may have different objects (e.g. datetime.datetime vs Timestamp) but will compare equal
+            if not Index(left.values).equals(Index(right.values)):
+                raise AssertionError(
+                    '[datetimelike_compat=True] {0} is not equal to {1}.'.format(left.values,
+                                                                                 right.values))
         else:
-            assert_attr_equal('name', left, right)
+            assert_numpy_array_equal(left.values, right.values)
+    else:
+        assert_almost_equal(left.get_values(), right.get_values(),
+                            check_less_precise, obj='{0}'.format(obj))
+
+    # metadata comparison
+    if check_names:
+        assert_attr_equal('name', left, right, obj=obj)
 
 
 # This could be refactored to use the NDFrame.equals method
@@ -709,19 +937,70 @@ def assert_frame_equal(left, right, check_dtype=True,
                        check_less_precise=False,
                        check_names=True,
                        by_blocks=False,
-                       check_exact=False):
-    if check_frame_type:
-        assert_isinstance(left, type(right))
-    assert_isinstance(left, DataFrame)
-    assert_isinstance(right, DataFrame)
+                       check_exact=False,
+                       check_datetimelike_compat=False,
+                       obj='DataFrame'):
 
-    if check_less_precise:
-        if not by_blocks:
-            assert_almost_equal(left.columns, right.columns)
-        assert_almost_equal(left.index, right.index)
-    else:
-        if not by_blocks:
-            assert_index_equal(left.columns, right.columns, check_names=check_names)
+    """Check that left and right DataFrame are equal.
+
+    Parameters
+    ----------
+    left : DataFrame
+    right : DataFrame
+    check_dtype : bool, default True
+        Whether to check the DataFrame dtype is identical.
+    check_index_type : bool, default False
+        Whether to check the Index class, dtype and inferred_type are identical.
+    check_column_type : bool, default False
+        Whether to check the columns class, dtype and inferred_type are identical.
+    check_frame_type : bool, default False
+        Whether to check the DataFrame class is identical.
+    check_less_precise : bool, default False
+        Specify comparison precision. Only used when check_exact is False.
+        5 digits (False) or 3 digits (True) after decimal points are compared.
+    check_names : bool, default True
+        Whether to check the Index names attribute.
+    by_blocks : bool, default False
+        Specify how to compare internal data. If False, compare by columns.
+        If True, compare by blocks.
+    check_exact : bool, default False
+        Whether to compare number exactly.
+    check_dateteimelike_compat : bool, default False
+        Compare datetime-like which is comparable ignoring dtype.
+    obj : str, default 'DataFrame'
+        Specify object name being compared, internally used to show appropriate
+        assertion message
+    """
+
+    # instance validation
+    assertIsInstance(left, DataFrame, '[DataFrame] ')
+    assertIsInstance(right, DataFrame, '[DataFrame] ')
+
+    if check_frame_type:
+        assertIsInstance(left, type(right))
+
+    # shape comparison (row)
+    if left.shape[0] != right.shape[0]:
+        raise_assert_detail(obj, 'DataFrame shape (number of rows) are different',
+                            '{0}, {1}'.format(left.shape[0], left.index),
+                            '{0}, {1}'.format(right.shape[0], right.index))
+    # shape comparison (columns)
+    if left.shape[1] != right.shape[1]:
+        raise_assert_detail(obj, 'DataFrame shape (number of columns) are different',
+                            '{0}, {1}'.format(left.shape[1], left.columns),
+                            '{0}, {1}'.format(right.shape[1], right.columns))
+
+    # index comparison
+    assert_index_equal(left.index, right.index, exact=check_index_type,
+                       check_names=check_names,
+                       check_less_precise=check_less_precise, check_exact=check_exact,
+                       obj='{0}.index'.format(obj))
+
+    # column comparison
+    assert_index_equal(left.columns, right.columns, exact=check_column_type,
+                       check_names=check_names,
+                       check_less_precise=check_less_precise, check_exact=check_exact,
+                       obj='{0}.columns'.format(obj))
 
     # compare by blocks
     if by_blocks:
@@ -730,35 +1009,23 @@ def assert_frame_equal(left, right, check_dtype=True,
         for dtype in list(set(list(lblocks.keys()) + list(rblocks.keys()))):
             assert dtype in lblocks
             assert dtype in rblocks
-            assert_frame_equal(lblocks[dtype],rblocks[dtype], check_dtype=check_dtype)
+            assert_frame_equal(lblocks[dtype], rblocks[dtype],
+                               check_dtype=check_dtype, obj='DataFrame.blocks')
 
     # compare by columns
     else:
         for i, col in enumerate(left.columns):
             assert col in right
-            lcol = left.icol(i)
-            rcol = right.icol(i)
+            lcol = left.iloc[:, i]
+            rcol = right.iloc[:, i]
             assert_series_equal(lcol, rcol,
                                 check_dtype=check_dtype,
                                 check_index_type=check_index_type,
                                 check_less_precise=check_less_precise,
                                 check_exact=check_exact,
-                                check_names=check_names)
-
-    if check_index_type:
-        for level in range(left.index.nlevels):
-            lindex = left.index.get_level_values(level)
-            rindex = right.index.get_level_values(level)
-            assert_isinstance(lindex, type(rindex))
-            assert_attr_equal('dtype', lindex, rindex)
-            assert_attr_equal('inferred_type', lindex, rindex)
-    if check_column_type:
-        assert_isinstance(left.columns, type(right.columns))
-        assert_attr_equal('dtype', left.columns, right.columns)
-        assert_attr_equal('inferred_type', left.columns, right.columns)
-    if check_names:
-        assert_attr_equal('names', left.index, right.index)
-        assert_attr_equal('names', left.columns, right.columns)
+                                check_names=check_names,
+                                check_datetimelike_compat=check_datetimelike_compat,
+                                obj='DataFrame.iloc[:, {0}]'.format(i))
 
 
 def assert_panelnd_equal(left, right,
@@ -767,7 +1034,7 @@ def assert_panelnd_equal(left, right,
                          assert_func=assert_frame_equal,
                          check_names=False):
     if check_panel_type:
-        assert_isinstance(left, type(right))
+        assertIsInstance(left, type(right))
 
     for axis in ['items', 'major_axis', 'minor_axis']:
         left_ind = getattr(left, axis)
@@ -817,62 +1084,89 @@ def getArangeMat():
 
 
 # make index
-def makeStringIndex(k=10):
-    return Index(rands_array(nchars=10, size=k))
+def makeStringIndex(k=10, name=None):
+    return Index(rands_array(nchars=10, size=k), name=name)
 
 
-def makeUnicodeIndex(k=10):
+def makeUnicodeIndex(k=10, name=None):
     return Index(randu_array(nchars=10, size=k))
 
-def makeCategoricalIndex(k=10, n=3):
+def makeCategoricalIndex(k=10, n=3, name=None):
     """ make a length k index or n categories """
     x = rands_array(nchars=4, size=n)
-    return CategoricalIndex(np.random.choice(x,k))
+    return CategoricalIndex(np.random.choice(x,k), name=name)
 
-def makeBoolIndex(k=10):
+def makeBoolIndex(k=10, name=None):
     if k == 1:
-        return Index([True])
+        return Index([True], name=name)
     elif k == 2:
-        return Index([False,True])
-    return Index([False,True] + [False]*(k-2))
+        return Index([False,True], name=name)
+    return Index([False,True] + [False]*(k-2), name=name)
 
-def makeIntIndex(k=10):
-    return Index(lrange(k))
+def makeIntIndex(k=10, name=None):
+    return Index(lrange(k), name=name)
 
-def makeFloatIndex(k=10):
+def makeFloatIndex(k=10, name=None):
     values = sorted(np.random.random_sample(k)) - np.random.random_sample(1)
-    return Index(values * (10 ** np.random.randint(0, 9)))
+    return Index(values * (10 ** np.random.randint(0, 9)), name=name)
 
-def makeDateIndex(k=10, freq='B'):
+def makeDateIndex(k=10, freq='B', name=None):
     dt = datetime(2000, 1, 1)
-    dr = bdate_range(dt, periods=k, freq=freq)
-    return DatetimeIndex(dr)
+    dr = bdate_range(dt, periods=k, freq=freq, name=name)
+    return DatetimeIndex(dr, name=name)
 
-def makeTimedeltaIndex(k=10, freq='D'):
-    return TimedeltaIndex(start='1 day',periods=k,freq=freq)
+def makeTimedeltaIndex(k=10, freq='D', name=None):
+    return TimedeltaIndex(start='1 day', periods=k, freq=freq, name=name)
 
-def makePeriodIndex(k=10):
+def makePeriodIndex(k=10, name=None):
     dt = datetime(2000, 1, 1)
-    dr = PeriodIndex(start=dt, periods=k, freq='B')
+    dr = PeriodIndex(start=dt, periods=k, freq='B', name=name)
     return dr
 
+def all_index_generator(k=10):
+    """Generator which can be iterated over to get instances of all the various
+    index classes.
+
+    Parameters
+    ----------
+    k: length of each of the index instances
+    """
+    all_make_index_funcs = [makeIntIndex, makeFloatIndex, makeStringIndex,
+                            makeUnicodeIndex, makeDateIndex, makePeriodIndex,
+                            makeTimedeltaIndex, makeBoolIndex,
+                            makeCategoricalIndex]
+    for make_index_func in all_make_index_funcs:
+        yield make_index_func(k=k)
+
+def all_timeseries_index_generator(k=10):
+    """Generator which can be iterated over to get instances of all the classes
+    which represent time-seires.
+
+    Parameters
+    ----------
+    k: length of each of the index instances
+    """
+    make_index_funcs = [makeDateIndex, makePeriodIndex, makeTimedeltaIndex]
+    for make_index_func in make_index_funcs:
+        yield make_index_func(k=k)
+
 
 # make series
-def makeFloatSeries():
+def makeFloatSeries(name=None):
     index = makeStringIndex(N)
-    return Series(randn(N), index=index)
+    return Series(randn(N), index=index, name=name)
 
 
-def makeStringSeries():
+def makeStringSeries(name=None):
     index = makeStringIndex(N)
-    return Series(randn(N), index=index)
+    return Series(randn(N), index=index, name=name)
 
 
-def makeObjectSeries():
+def makeObjectSeries(name=None):
     dateIndex = makeDateIndex(N)
     dateIndex = Index(dateIndex, dtype=object)
     index = makeStringIndex(N)
-    return Series(dateIndex, index=index)
+    return Series(dateIndex, index=index, name=name)
 
 
 def getSeriesData():
@@ -880,16 +1174,16 @@ def getSeriesData():
     return dict((c, Series(randn(N), index=index)) for c in getCols(K))
 
 
-def makeTimeSeries(nper=None, freq='B'):
+def makeTimeSeries(nper=None, freq='B', name=None):
     if nper is None:
         nper = N
-    return Series(randn(nper), index=makeDateIndex(nper, freq=freq))
+    return Series(randn(nper), index=makeDateIndex(nper, freq=freq), name=name)
 
 
-def makePeriodSeries(nper=None):
+def makePeriodSeries(nper=None, name=None):
     if nper is None:
         nper = N
-    return Series(randn(nper), index=makePeriodIndex(nper))
+    return Series(randn(nper), index=makePeriodIndex(nper), name=name)
 
 
 def getTimeSeriesData(nper=None, freq='B'):
@@ -1313,7 +1607,8 @@ _network_error_messages = (
     'HTTP Error 403',
     'Temporary failure in name resolution',
     'Name or service not known',
-    )
+    'Connection refused',
+)
 
 # or this e.errno/e.reason.errno
 _network_errno_vals = (
@@ -1665,7 +1960,8 @@ class _AssertRaisesContextmanager(object):
 
 
 @contextmanager
-def assert_produces_warning(expected_warning=Warning, filter_level="always", clear=None):
+def assert_produces_warning(expected_warning=Warning, filter_level="always",
+                            clear=None, check_stacklevel=True):
     """
     Context manager for running code that expects to raise (or not raise)
     warnings.  Checks that code raises the expected warning and only the
@@ -1713,6 +2009,16 @@ def assert_produces_warning(expected_warning=Warning, filter_level="always", cle
             if (expected_warning and issubclass(actual_warning.category,
                                                 expected_warning)):
                 saw_warning = True
+
+                if check_stacklevel and issubclass(actual_warning.category,
+                                                   (FutureWarning, DeprecationWarning)):
+                    from inspect import getframeinfo, stack
+                    caller = getframeinfo(stack()[2][0])
+                    msg = ("Warning not set with correct stacklevel. File were warning"
+                           " is raised: {0} != {1}. Warning message: {2}".format(
+                               actual_warning.filename, caller.filename,
+                               actual_warning.message))
+                    assert actual_warning.filename == caller.filename, msg
             else:
                 extra_warnings.append(actual_warning.category.__name__)
         if expected_warning:
@@ -1722,24 +2028,6 @@ def assert_produces_warning(expected_warning=Warning, filter_level="always", cle
                                     % extra_warnings)
 
 
-def skip_if_no_ne(engine='numexpr'):
-    import nose
-    _USE_NUMEXPR = pd.computation.expressions._USE_NUMEXPR
-
-    if engine == 'numexpr':
-        try:
-            import numexpr as ne
-        except ImportError:
-            raise nose.SkipTest("numexpr not installed")
-
-        if not _USE_NUMEXPR:
-            raise nose.SkipTest("numexpr disabled")
-
-        if ne.__version__ < LooseVersion('2.0'):
-            raise nose.SkipTest("numexpr version too low: "
-                                "%s" % ne.__version__)
-
-
 def disabled(t):
     t.disabled = True
     return t
@@ -1790,3 +2078,55 @@ def use_numexpr(use, min_elements=expr._MIN_ELEMENTS):
 for name, obj in inspect.getmembers(sys.modules[__name__]):
     if inspect.isfunction(obj) and name.startswith('assert'):
         setattr(TestCase, name, staticmethod(obj))
+
+
+def test_parallel(num_threads=2, kwargs_list=None):
+    """Decorator to run the same function multiple times in parallel.
+
+    Parameters
+    ----------
+    num_threads : int, optional
+        The number of times the function is run in parallel.
+    kwargs_list : list of dicts, optional
+        The list of kwargs to update original function kwargs on different threads.
+    Notes
+    -----
+    This decorator does not pass the return value of the decorated function.
+
+    Original from scikit-image: https://github.com/scikit-image/scikit-image/pull/1519
+
+    """
+
+    assert num_threads > 0
+    has_kwargs_list = kwargs_list is not None
+    if has_kwargs_list:
+        assert len(kwargs_list) == num_threads
+    import threading
+
+    def wrapper(func):
+        @wraps(func)
+        def inner(*args, **kwargs):
+            if has_kwargs_list:
+                update_kwargs = lambda i: dict(kwargs, **kwargs_list[i])
+            else:
+                update_kwargs = lambda i: kwargs
+            threads = []
+            for i in range(num_threads):
+                updated_kwargs = update_kwargs(i)
+                thread = threading.Thread(target=func, args=args,
+                                          kwargs=updated_kwargs)
+                threads.append(thread)
+            for thread in threads:
+                thread.start()
+            for thread in threads:
+                thread.join()
+        return inner
+    return wrapper
+
+
+class SubclassedDataFrame(DataFrame):
+    _metadata = ['testattr']
+
+    @property
+    def _constructor(self):
+        return SubclassedDataFrame
